"""
Constants for csle collector
"""
import re

LATEST_VERSION = "latest"
INSTALL = "sudo /root/miniconda3/bin/pip install -U --no-cache-dir csle-collector "


class DICT_PROPERTIES:
    """
    String constants representing dict properties
    """
    RUNNING = "running"
    TOPICS = "topics"


class HTTP:
    """
    Constants related to HTTP
    """
    HTTP_PROTOCOL_PREFIX = "http://"
    LOCALHOST = "localhost"
    GET = "GET"
    PUT = "PUT"
    POST = "POST"
    DELETE = "DELETE"
    APPLICATION_JSON_TYPE = "application/json"
    OK_RESPONSE_CODE = 200
    INTERNAL_SERVER_ERROR_RESPONSE_CODE = 500
    BAD_REQUEST_RESPONSE_CODE = 400


class BEATS:
    """
    Constants related to beats
    """
    RELOAD_ENABLED_PROPERTY = "reload.enabled"
    SETUP_TEMPLATE_SETTINGS_PROPERTY = "setup.template.settings"
    INDEX_NUM_SHARDS_PROPERTY = "index.number_of_shards"
    SETUP_KIBANA_PROPERTY = "setup.kibana"
    HOST_PROPERTY = "host"
    HOSTS_PROPERTY = "hosts"
    ELASTIC_OUTPUT_PROPERTY = "output.elasticsearch"
    PROCESSORS_PROPERTY = "processors"
    ADD_HOST_METADATA_PROPERTY = "add_host_metadata"
    WHEN_NOT_CONTAIN_TAGS_PROPERTY = "when.not.contains.tags"
    FORWARDED_PROPERTY = "forwarded"
    MODULE_PROPERTY = "module"
    LOG_PROPERTY = "log"
    SYSLOG_PROPERTY = "syslog"
    AUTH_PROPERTY = "auth"
    SLOWLOG_PROPERTY = "slowlog"
    AUDIT_PROPERTY = "audit"
    SERVER_PROPERTY = "server"
    VAR_INPUT_PROPERTY = "var.input"
    VAR_PATHS_PROPERTY = "var.paths"
    FILE_PROPERTY = "file"
    TYPE_PROPERTY = "type"
    FILESTREAM_PROPERTY = "filestream"
    KAFKA_PROPERTY = "kafka"
    TOPICS_PROPERTY = "topics"
    GROUP_ID_PROPERTY = "group_id"
    ID_PROPERTY = "id"
    NAME_PROPERTY = "name"
    ENABLED_PROPERTY = "enabled"
    PATHS_PROPERTY = "paths"
    PATH_PROPERTY = "path"
    PERIOD_PROPERTY = "period"
    METRICSETS_PROPERTY = "metricsets"


class FILEBEAT:
    """
    Constants related to Filebeat
    """
    FILEBEAT_GROUP_ID = "filebeat"
    CONFIG_DIR = "/etc/filebeat/"
    CONFIG_FILE = "/etc/filebeat/filebeat.yml"
    SETUP_CMD = "filebeat setup -e"
    SNORT_MODULE = "snort"
    ELASTICSEARCH_MODULE = "elasticsearch"
    KIBANA_MODULE = "kibana"
    SYSTEM_MODULE = "system"
    KAFKA_MODULE = "kafka"
    LOGSTASH_MODULE = "logstash"
    ENABLE_MODULE_CMD = "filebeat modules enable {}"
    MODULES_CONFIG_DIR = "/etc/filebeat/modules.d/"
    SNORT_MODULE_CONFIG_FILE = "snort.yml"
    LOGSTASH_MODULE_CONFIG_FILE = "logstash.yml"
    KIBANA_MODULE_CONFIG_FILE = "kibana.yml"
    SYSTEM_MODULE_CONFIG_FILE = "system.yml"
    KAFKA_MODULE_CONFIG_FILE = "kafka.yml"
    ELASTICSEARCH_MODULE_CONFIG_FILE = "elasticsearch.yml"
    INPUTS_PROPERTY = "filebeat.inputs"
    MODULES_PROPERTY = "filebeat.config.modules"
    FILEBEAT_STATUS = "sudo service filebeat status"
    FILEBEAT_START = "sudo service filebeat start"
    FILEBEAT_STOP = "sudo service filebeat stop"


class PACKETBEAT:
    """
    Constants related to Packetbeat
    """
    FILEBEAT_GROUP_ID = "packetbeat"
    CONFIG_DIR = "/etc/packetbeat/"
    CONFIG_FILE = "/etc/packetbeat/packetbeat.yml"
    SETUP_CMD = "packetbeat setup -e"
    PACKETBEAT_STATUS = "sudo service packetbeat status"
    PACKETBEAT_START = "sudo service packetbeat start"
    PACKETBEAT_STOP = "sudo service packetbeat stop"
    INTERFACES_TYPE_PROPERTY = "packetbeat.interfaces.type"
    AF_PACKET_PROPERTY = "af_packet"
    ANY_DEVICE_PROPERTY = "any"
    INTERFACES_DEVICE_PROPERTY = "packetbeat.interfaces.device"
    FLOWS = "packetbeat.flows"
    TIMEOUT_PROPERTY = "timeout"
    PROTOCOLS = "packetbeat.protocols"
    ICMP_PROTOCOL = "icmp"
    AMQP_PROTOCOL = "amqp"
    PORTS_PROPERTY = "ports"
    AMQP_PORTS = [5672]
    CASSANDRA_PROTOCOL = "cassandra"
    CASSANDRA_PORTS = [9042]
    DHCPV4_PROTOCOL = "dhcpv4"
    DHCPV4_PORTS = [67, 68]
    DNS_PROTOCOL = "dns"
    DNS_PORTS = [53]
    HTTP_PROTOCOL = "http"
    HTTP_PORTS = [80, 8080, 8000, 5000, 8002]
    MEMCACHE_PROTOCOL = "memcache"
    MEMCACHE_PORTS = [11211]
    MYSQL_PROTOCOL = "mysql"
    MYSQL_PORTS = [3306, 3307]
    PGSQL_PROTOCOL = "pgsql"
    PGSQL_PORTS = [5432]
    REDIS_PROTOCOL = "redis"
    REDIS_PORTS = [6379]
    THRIFT_PROTOCOL = "thrift"
    THRIFT_PORTS = [9090]
    MONGODB_PROTOCOL = "mongodb"
    MONGODB_PORTS = [27017]
    NFS_PROTOCOL = "nfs"
    NFS_PORTS = [2049]
    TLS_PROTOCOL = "tls"
    TLS_PORTS = [443, 993, 995, 5223, 8443, 8883, 9243]
    SIP_PROTOCOL = "sip"
    SIP_PORTS = [9243]


class METRICBEAT:
    """
    Constants related to Metricbeat
    """
    CONFIG_DIR = "/etc/metricbeat/"
    CONFIG_FILE = "/etc/metricbeat/metricbeat.yml"
    SETUP_CMD = "metricbeat setup -e"
    ELASTICSEARCH_MODULE = "elasticsearch"
    KIBANA_MODULE = "kibana"
    SYSTEM_MODULE = "system"
    LINUX_MODULE = "linux"
    KAFKA_MODULE = "kafka"
    LOGSTASH_MODULE = "logstash"
    ENABLE_MODULE_CMD = "metricbeat modules enable {}"
    MODULES_CONFIG_DIR = "/etc/metricbeat/modules.d/"
    SNORT_MODULE_CONFIG_FILE = "snort.yml"
    LOGSTASH_MODULE_CONFIG_FILE = "logstash.yml"
    KIBANA_MODULE_CONFIG_FILE = "kibana.yml"
    SYSTEM_MODULE_CONFIG_FILE = "system.yml"
    LINUX_MODULE_CONFIG_FILE = "linux.yml"
    KAFKA_MODULE_CONFIG_FILE = "kafka.yml"
    ELASTICSEARCH_MODULE_CONFIG_FILE = "elasticsearch.yml"
    MODULES_PROPERTY = "metricbeat.config.modules"
    METRICBEAT_STATUS = "sudo service metricbeat status"
    METRICBEAT_START = "sudo service metricbeat start"
    METRICBEAT_STOP = "sudo service metricbeat stop"
    PROCESSES_PROPERTY = "processes"
    CPU_METRICS_PROPERTY = "cpu.metrics"
    CORE_METRICS_PROPERTY = "core.metrics"
    PERCENTAGES_PROPERTY = "percentages"
    NORMALIZED_PERCENTAGES_PROPERTY = "normalized_percentages"
    CPU_METRIC = "cpu"
    LOAD_METRIC = "load"
    MEMORY_METRIC = "memory"
    NETWORK_METRIC = "network"
    PROCESS_METRIC = "process"
    PROCESS_SUMMARY_METRIC = "process_summary"
    SOCKET_SUMMARY_METRIC = "socket_summary"
    PAGEINFO_METRIC = "pageinfo"
    SUMMARY_METRIC = "memory"


class HEARTBEAT:
    """
    Constants related to heartbeat
    """
    CONFIG_DIR = "/etc/heartbeat/"
    CONFIG_FILE = "/etc/heartbeat/heartbeat.yml"
    SETUP_CMD = "heartbeat setup -e"
    HEARTBEAT_STATUS = "sudo service heartbeat-elastic status"
    HEARTBEAT_START = "sudo service heartbeat-elastic start"
    HEARTBEAT_STOP = "sudo service heartbeat-elastic stop"
    SCHEDULE_PROPERTY = "schedule"
    ICMP_MONITOR_TYPE = "icmp"
    HEARTBEAT_MONITORS_PROPERTY = "heartbeat.monitors"
    CSLE_MONITOR_SERVICE_NAME = "csle-topology-connection-service"
    CSLE_MONITOR_SERVICE_ID = "csle-topology-connection-service-id"


class GRPC:
    """
    Constants related to GRPC
    """
    TIMEOUT_SECONDS = 30
    CONFIG_TIMEOUT_SECONDS = 300
    OPERATION_TIMEOUT_SECONDS = 30000


class SYSTEM:
    """
    Constants related to system
    """
    AUTH_LOG = "/var/log/auth.log"
    SYSLOG = "/var/log/syslog"


class LOG_FILES:
    """
    Constants related to the log files
    """
    KAFKA_MANAGER_LOG_FILE = "kafka_manager.log"
    KAFKA_MANAGER_LOG_DIR = "/"
    OSSEC_IDS_MANAGER_LOG_FILE = "ossec_ids_manager.log"
    OSSEC_IDS_MANAGER_LOG_DIR = "/"
    SNORT_IDS_MANAGER_LOG_FILE = "snort_ids_manager.log"
    SNORT_IDS_MANAGER_LOG_DIR = "/"
    ELK_MANAGER_LOG_FILE = "elk_manager.log"
    ELK_MANAGER_LOG_DIR = "/"
    DOCKER_STATS_MANAGER_LOG_FILE = "docker_stats_manager.log"
    CLUSTER_MANAGER_LOG_FILE = "cluster_manager.log"
    DOCKER_STATS_MANAGER_LOG_DIR = "/var/log/csle/"
    CLUSTER_MANAGER_LOG_DIR = "/var/log/csle/"
    HOST_MANAGER_LOG_FILE = "host_manager.log"
    HOST_MANAGER_LOG_DIR = "/"
    CLIENT_MANAGER_LOG_FILE = "client_manager.log"
    CLIENT_MANAGER_LOG_DIR = "/"
    TRAFFIC_MANAGER_LOG_FILE = "traffic_manager.log"
    TRAFFIC_MANAGER_LOG_DIR = "/"
    KAFKA_LOG_FILE = "/usr/local/kafka/logs/server.log"
    RYU_MANAGER_LOG_FILE = "ryu_manager.log"
    RYU_MANAGER_LOG_DIR = "/"
    DEFAULT_LOG_FILE_PATHS = ["/*.log", "/var/log/*.log", "/var/log/*/*.log", "/var/log/*/*/*.log"]


class MANAGER_PORTS:
    """
    Constants related to the ports of managers
    """
    TRAFFIC_MANAGER_DEFAULT_PORT = 50043
    CLIENT_MANAGER_DEFAULT_PORT = 50044
    SDN_CONTROLLER_MANAGER_DEFAULT_PORT = 50042
    HOST_MANAGER_DEFAULT_PORT = 50049
    SNORT_IDS_MANAGER_DEFAULT_PORT = 50048
    OSSEC_IDS_MANAGER_DEFAULT_PORT = 50047
    DOCKER_STATS_MANAGER_DEFAULT_PORT = 50046
    ELK_MANAGER_DEFAULT_PORT = 50045
    KAFKA_MANAGER_DEFAULT_PORT = 50051


class GRPC_WORKERS:
    """
    Constants related to the number of GRPC workers
    """
    DEFAULT_MAX_NUM_WORKERS = 10


class TRAFFIC_GENERATOR:
    """
    Constants related to the traffic generator
    """
    START_TRAFFIC_GENERATOR_CMD = "sudo nohup /traffic_generator.sh &"
    TRAFFIC_GENERATOR_FILE_NAME = "traffic_generator.sh"
    CREATE_TRAFFIC_GENERATOR_FILE = "sudo touch /traffic_generator.sh"
    MAKE_TRAFFIC_GENERATOR_FILE_EXECUTABLE = "sudo chmod 777 /traffic_generator.sh"
    REMOVE_OLD_TRAFFIC_GENERATOR_FILE = "sudo rm -f /traffic_generator.sh"
    CHECK_IF_TRAFFIC_GENERATOR_IS_RUNNING = "ps -aux | grep traffic_generator"
    STOP_TRAFFIC_GENERATOR = "sudo pkill -f traffic_generator.sh"


class DOCKER_STATS:
    """
    Constants related to Docker stats
    """
    CPU_STATS = "cpu_stats"
    CPU_USAGE = "cpu_usage"
    PERCPU_USAGE = "percpu_usage"
    PRECPU_STATS = "precpu_stats"
    TOTAL_USAGE = "total_usage"
    SYSTEM_CPU_USAGE = "system_cpu_usage"
    ONLINE_CPUS = "online_cpus"
    BLKIO_STATS = "blkio_stats"
    IO_SERVICE_BYTES_RECURSIVE = "io_service_bytes_recursive"
    OP = "op"
    READ = "Read"
    VALUE = "value"
    WRITE = "Write"
    NETWORKS = "networks"
    RX_BYTES = "rx_bytes"
    TX_BYTES = "tx_bytes"
    MEMORY_STATS = "memory_stats"
    USAGE = "usage"
    LIMIT = "limit"
    PIDS = "pids"
    TIMESTAMP = "timestamp"
    CPU_PERCENT = "cpu_percent"
    MEM_CURRENT = "mem_current"
    MEM_TOTAL = "mem_total"
    MEM_PERCENT = "mem_percent"
    BLK_READ = "blk_read"
    BLK_WRITE = "blk_write"
    NET_RX = "net_rx"
    NET_TX = "net_tx"
    PIDS_STATS = "pids_stats"
    CURRENT = "current"
    CONTAINER_NAME = "container_name"
    CONTAINER_ID = "container_id"
    CONTAINER_IP = "container_ip"
    UNIX_DOCKER_SOCK_URL = "unix://var/run/docker.sock"


class OSSEC:
    """
    Constants related to the OSSEC HIDS
    """
    MAX_ALERTS = 10000
    OSSEC_ALERTS_FILE = "/var/ossec/alerts/alerts.log"
    OSSEC_LOG_FILE = "/var/ossec/logs/ossec.log"
    TAIL_ALERTS_COMMAND = "sudo tail -" + str(MAX_ALERTS)
    ALERTLINE_REGEX = re.compile(r"\*\* Alert (\d+.\d+)*: - (\w+.+)")
    HOSTLINE_REGEX = re.compile(r"\d+ \w+ \d+ \d+:\d+:\d+ \((\w+.+)\) (\d+.\d+.\d+.\d+)")
    SERVHOSTLINE_REGEX = re.compile(r"\d+ \w+ \d+ \d+:\d+:\d+ (\w+)")
    RULELINE_REGEX = re.compile(r"Rule: (\d+)* \(level (\d+)\) -> '(\w+.+)'")
    SRCIPLINE_REGEX = re.compile(r"Src IP: (\d+.\d+.\d+.\d+)")
    USERLINE_REGEX = re.compile(r"User: (\w+)")
    DATELINEREGEX = re.compile(r"\d+ \w+ \d+ \d+:\d+:\d+")
    OSSEC_ALERT_RULE_ID_TO_DESCR = {}
    OSSEC_ALERT_RULE_ID_TO_DESCR[0] = "Ignored - No action taken. Used to avoid false positives. " \
                                      "These rules are scanned before all the others. " \
                                      "They include events with no security relevance."
    OSSEC_ALERT_RULE_ID_TO_DESCR[1] = "None"
    OSSEC_ALERT_RULE_ID_TO_DESCR[2] = "System low priority notification - System notification or status messages. " \
                                      "They have no security relevance."
    OSSEC_ALERT_RULE_ID_TO_DESCR[3] = "Successful/Authorized events - They include successful login attempts, " \
                                      "firewall allow events, etc."
    OSSEC_ALERT_RULE_ID_TO_DESCR[4] = "System low priority error - Errors related to bad configurations or " \
                                      "unused devices/applications. They have no security relevance and are usually " \
                                      "caused by default installations or software testing."
    OSSEC_ALERT_RULE_ID_TO_DESCR[5] = "User generated error - They include missed passwords, denied actions, etc. " \
                                      "By itself they have no security relevance."
    OSSEC_ALERT_RULE_ID_TO_DESCR[6] = "Low relevance attack - They indicate a worm or a virus that have no affect to " \
                                      "the system (like code red for apache servers, etc). " \
                                      "They also include frequently IDS events and frequently errors."
    OSSEC_ALERT_RULE_ID_TO_DESCR[7] = "'Bad word' matching. They include words like 'bad', " \
                                      "'error', etc. These events " \
                                      "are most of the time unclassified and may have some security relevance."
    OSSEC_ALERT_RULE_ID_TO_DESCR[8] = "First time seen - Include first time seen events. First time an IDS event is " \
                                      "fired or the first time an user logged in. If you just started using OSSEC " \
                                      "HIDS these messages will probably be frequently. After a while they should " \
                                      "go away, It also includes security relevant actions " \
                                      "(like the starting of a sniffer or something like that)."
    OSSEC_ALERT_RULE_ID_TO_DESCR[9] = "Error from invalid source - Include attempts to login as an unknown user or " \
                                      "from an invalid source. May have security relevance (specially if repeated). " \
                                      "They also include errors regarding the “admin” (root) account."
    OSSEC_ALERT_RULE_ID_TO_DESCR[10] = "Multiple user generated errors - They include multiple bad passwords, " \
                                       "multiple failed logins, etc. " \
                                       "They may indicate an attack or may just be that a user " \
                                       "just forgot his credentials."
    OSSEC_ALERT_RULE_ID_TO_DESCR[11] = "Integrity checking warning - They include messages regarding the " \
                                       "modification of binaries or the presence of rootkits (by rootcheck). " \
                                       "If you just modified your system configuration you should be fine " \
                                       "regarding the “syscheck” messages. " \
                                       "They may indicate a successful attack. Also included IDS events that " \
                                       "will be ignored (high number of repetitions)."
    OSSEC_ALERT_RULE_ID_TO_DESCR[12] = "High importancy event - They include error or warning messages from the " \
                                       "system, kernel, etc. They may indicate an attack against a " \
                                       "specific application."
    OSSEC_ALERT_RULE_ID_TO_DESCR[13] = "Unusual error (high importance) - Most of the times it matches a " \
                                       "common attack pattern."
    OSSEC_ALERT_RULE_ID_TO_DESCR[14] = "High importance security event. Most of the times done with correlation and " \
                                       "it indicates an attack."
    OSSEC_ALERT_RULE_ID_TO_DESCR[15] = "Severe attack - No chances of false positives. Immediate attention " \
                                       "is necessary."
    OSSEC_IDS_ALERT_GROUP_ID = {}
    OSSEC_IDS_ALERT_GROUP_ID["invalid_login"] = 0
    OSSEC_IDS_ALERT_GROUP_ID["authentication_success"] = 1
    OSSEC_IDS_ALERT_GROUP_ID["authentication_failed"] = 2
    OSSEC_IDS_ALERT_GROUP_ID["connection_attempt"] = 3
    OSSEC_IDS_ALERT_GROUP_ID["attacks"] = 4
    OSSEC_IDS_ALERT_GROUP_ID["adduser"] = 5
    OSSEC_IDS_ALERT_GROUP_ID["sshd"] = 6
    OSSEC_IDS_ALERT_GROUP_ID["ids"] = 7
    OSSEC_IDS_ALERT_GROUP_ID["firewall"] = 8
    OSSEC_IDS_ALERT_GROUP_ID["squid"] = 9
    OSSEC_IDS_ALERT_GROUP_ID["apache"] = 10
    OSSEC_IDS_ALERT_GROUP_ID["syslog"] = 11
    OSSEC_SEVERE_ALERT_LEVEL_THRESHOLD = 10
    STOP_OSSEC_IDS = "/var/ossec/bin/ossec-control stop"
    START_OSSEC_IDS = "/var/ossec/bin/ossec-control start"
    CHECK_IF_OSSEC_IS_RUNNING_CMD = "service ossec status"
    OSSEC_RUNNING_SEARCH = "ossec-execd is running"


class SNORT_IDS_ROUTER:
    """
    Constants related to the Snort IDS
    """
    MAX_ALERTS = 10000
    UPDATE_RULESET = "/pulledpork/pulledpork.pl -c /pulledpork/etc/pulledpork.conf -l -P -E -H SIGHUP"
    SNORT_FAST_LOG_FILE = "/var/snort/fast.log"
    SNORT_ALERTS_FILE = "/var/snort/alert.csv"
    SNORT_STATS_FILE = "/var/snort/snort.stats"
    TAIL_ALERTS_COMMAND = "sudo tail -" + str(MAX_ALERTS)
    TAIL_FAST_LOG_COMMAND = "sudo tail -" + str(str(MAX_ALERTS))
    TAIL_ALERTS_LATEST_COMMAND = "sudo tail -1"
    PRIORITY_REGEX = re.compile(r"Priority: \d")
    IPS_REGEX = re.compile(r"\d{1,3}.\d{1,3}\.\d{1,3}\.\d{1,3} -> \d{1,3}.\d{1,3}\.\d{1,3}\.\d{1,3}")
    RULE_ID_REGEX = re.compile(r"\[\d{1,3}\:\d{1,3}")
    CLASSIFICATION_REGEX = re.compile(r"(?<=Classification: )(.*?)(?=])")
    SNORT_SEVERE_ALERT_PRIORITY_THRESHOLD = 2
    SNORT_ALERT_IDS_ID = {}
    SNORT_ALERT_IDS_ID["tcp-connection"] = 0
    SNORT_ALERT_IDS_ID["A TCP connection was detected"] = 0
    SNORT_ALERT_IDS_ID["unknown"] = 1
    SNORT_ALERT_IDS_ID["Unknown Traffic"] = 1
    SNORT_ALERT_IDS_ID["string-detect"] = 2
    SNORT_ALERT_IDS_ID["A suspicious string was detected"] = 2
    SNORT_ALERT_IDS_ID["protocol-command-decode"] = 3
    SNORT_ALERT_IDS_ID["Generic Protocol Command Decode"] = 3
    SNORT_ALERT_IDS_ID["not-suspicious"] = 4
    SNORT_ALERT_IDS_ID["Not Suspicious Traffic"] = 4
    SNORT_ALERT_IDS_ID["network-scan"] = 5
    SNORT_ALERT_IDS_ID["Detection of a Network Scan"] = 5
    SNORT_ALERT_IDS_ID["misc-activity"] = 6
    SNORT_ALERT_IDS_ID["Misc activity"] = 6
    SNORT_ALERT_IDS_ID["icmp-event"] = 7
    SNORT_ALERT_IDS_ID["Generic ICMP event"] = 7
    SNORT_ALERT_IDS_ID["web-application-activity"] = 8
    SNORT_ALERT_IDS_ID["Access to a potentially vulnerable web application"] = 8
    SNORT_ALERT_IDS_ID["unusual-client-port-connection"] = 9
    SNORT_ALERT_IDS_ID["A client was using an unusual port"] = 9
    SNORT_ALERT_IDS_ID["system-call-detect"] = 10
    SNORT_ALERT_IDS_ID["A system call was detected"] = 10
    SNORT_ALERT_IDS_ID["suspicious-login"] = 11
    SNORT_ALERT_IDS_ID["An attempted login using a suspicious username was detected"] = 11
    SNORT_ALERT_IDS_ID["suspicious-filename-detect"] = 12
    SNORT_ALERT_IDS_ID["A suspicious filename was detected"] = 12
    SNORT_ALERT_IDS_ID["successful-recon-limited"] = 13
    SNORT_ALERT_IDS_ID["Information Leak"] = 13
    SNORT_ALERT_IDS_ID["successful-recon-largescale"] = 14
    SNORT_ALERT_IDS_ID["Large Scale Information Leak"] = 14
    SNORT_ALERT_IDS_ID["successful-dos"] = 15
    SNORT_ALERT_IDS_ID["Denial of Service"] = 15
    SNORT_ALERT_IDS_ID["rpc-portmap-decode"] = 16
    SNORT_ALERT_IDS_ID["Decode of an RPC Query"] = 16
    SNORT_ALERT_IDS_ID["non-standard-protocol"] = 17
    SNORT_ALERT_IDS_ID["Detection of a non-standard protocol or event"] = 17
    SNORT_ALERT_IDS_ID["misc-attack"] = 18
    SNORT_ALERT_IDS_ID["Misc Attack"] = 18
    SNORT_ALERT_IDS_ID["denial-of-service"] = 19
    SNORT_ALERT_IDS_ID["Detection of a Denial of Service Attack"] = 19
    SNORT_ALERT_IDS_ID["default-login-attempt"] = 20
    SNORT_ALERT_IDS_ID["Attempt to login by a default username and password"] = 20
    SNORT_ALERT_IDS_ID["bad-unknown"] = 21
    SNORT_ALERT_IDS_ID["Potentially Bad Traffic"] = 21
    SNORT_ALERT_IDS_ID["attempted-recon"] = 22
    SNORT_ALERT_IDS_ID["Attempted Information Leak"] = 22
    SNORT_ALERT_IDS_ID["attempted-dos"] = 23
    SNORT_ALERT_IDS_ID["Attempted Denial of Service"] = 23
    SNORT_ALERT_IDS_ID["web-application-attack"] = 24
    SNORT_ALERT_IDS_ID["Web Application Attack"] = 24
    SNORT_ALERT_IDS_ID["unsuccessful-user"] = 25
    SNORT_ALERT_IDS_ID["Unsuccessful User Privilege Gain"] = 25
    SNORT_ALERT_IDS_ID["trojan-activity"] = 26
    SNORT_ALERT_IDS_ID["A Network Trojan was detected"] = 26
    SNORT_ALERT_IDS_ID["successful-user"] = 27
    SNORT_ALERT_IDS_ID["Successful User Privilege Gain"] = 27
    SNORT_ALERT_IDS_ID["successful-admin"] = 28
    SNORT_ALERT_IDS_ID["Successful Administrator Privilege Gain"] = 28
    SNORT_ALERT_IDS_ID["shellcode-detect"] = 29
    SNORT_ALERT_IDS_ID["Executable code was detected"] = 29
    SNORT_ALERT_IDS_ID["policy-violation"] = 30
    SNORT_ALERT_IDS_ID["Potential Corporate Privacy Violation"] = 30
    SNORT_ALERT_IDS_ID["inappropriate-content"] = 31
    SNORT_ALERT_IDS_ID["Inappropriate Content was Detected"] = 31
    SNORT_ALERT_IDS_ID["attempted-user"] = 32
    SNORT_ALERT_IDS_ID["Attempted User Privilege Gain"] = 32
    SNORT_ALERT_IDS_ID["attempted-admin"] = 33
    SNORT_ALERT_IDS_ID["Attempted Administrator Privilege Gain"] = 33
    STOP_SNORT_IDS = "kill -9 $(pgrep snort)"
    START_SNORT_IDS = "sudo snort -D -q -c /etc/snort/snort.conf -i {}:{} -l " \
                      "/var/snort/ -h {} -Q -I --create-pidfile"
    SNORT_LOG_DIR_PERMISSION_CMD = "sudo chmod -R 777 /var/snort"
    CHECK_IF_SNORT_IS_RUNNING_CMD = "ps -aux | grep snort.conf"
    PS_AUX_CMD = "ps -aux"
    GREP_SNORT_CONF = "grep snort.conf"
    SEARCH_SNORT_RUNNING = "/etc/snort/snort.conf"


class HOST_METRICS:
    """
    Constants related to the defender's sensor commands
    """
    LIST_LOGGED_IN_USERS_CMD = "users"
    LIST_OPEN_CONNECTIONS_CMD = "netstat -n"
    LIST_USER_ACCOUNTS = "cat /etc/passwd"
    LIST_FAILED_LOGIN_ATTEMPTS = "sudo tail -10000 /var/log/auth.log"
    LIST_SUCCESSFUL_LOGIN_ATTEMPTS = "last"
    LIST_NUMBER_OF_PROCESSES = "ps -e | wc -l"


class EXTERNAL_NETWORK:
    """
    Constants related to the external network
    """
    NETWORK_ID_THIRD_OCTET = 1


class ELK_CONFIG:
    """
    Constants related to the ELK container configuration
    """
    NETWORK_ID_THIRD_OCTET = 253
    NETWORK_ID_FOURTH_OCTET = 252
    SUFFIX = "_1"


class KAFKA_CONFIG:
    """
    Constants related to the kafka container configuration
    """
    DEFAULT_NUM_PARTITIONS = 1
    DEFAULT_NUM_REPLICAS = 1
    DEFAULT_RETENTION_TIME_HOURS = 240
    NETWORK_ID_THIRD_OCTET = 253
    NETWORK_ID_FOURTH_OCTET = 253
    SUFFIX = "_1"
    CLIENT_POPULATION_TOPIC_NAME = "client_population"
    SNORT_IDS_LOG_TOPIC_NAME = "snort_ids_log"
    SNORT_IDS_RULE_LOG_TOPIC_NAME = "snort_ids_rule_log"
    SNORT_IDS_IP_LOG_TOPIC_NAME = "snort_ids_ip_log"
    OSSEC_IDS_LOG_TOPIC_NAME = "ossec_ids_log"
    HOST_METRICS_TOPIC_NAME = "host_metrics"
    DOCKER_STATS_TOPIC_NAME = "docker_stats"
    DOCKER_HOST_STATS_TOPIC_NAME = "docker_host_stats"
    OPENFLOW_FLOW_STATS_TOPIC_NAME = "openflow_flow_stats"
    OPENFLOW_PORT_STATS_TOPIC_NAME = "openflow_port_stats"
    OPENFLOW_AGG_FLOW_STATS_TOPIC_NAME = "openflow_flow_agg_stats"
    AVERAGE_OPENFLOW_FLOW_STATS_PER_SWITCH_TOPIC_NAME = "avg_openflow_flow_stats_per_switch"
    AVERAGE_OPENFLOW_PORT_STATS_PER_SWITCH_TOPIC_NAME = "avg_openflow_port_stats_per_switch"
    ATTACKER_ACTIONS_TOPIC_NAME = "attacker_actions"
    DEFENDER_ACTIONS_TOPIC_NAME = "defender_actions"
    CLIENT_POPULATION_TOPIC_ATTRIBUTES = ["timestamp", "ip", "num_clients", "rate"]
    SNORT_IDS_LOG_TOPIC_ATTRIBUTES = ["timestamp", "ip", "attempted-admin", "attempted-user",
                                      "inappropriate-content", "policy-violation", "shellcode-detect",
                                      "successful-admin", "successful-user", "trojan-activity", "unsuccessful-user",
                                      "web-application-attack", "attempted-dos", "attempted-recon", "bad-unknown",
                                      "default-login-attempt", "denial-of-service", "misc-attack",
                                      "non-standard-protocol", "rpc-portmap-decode", "successful-dos",
                                      "successful-recon-largescale", "successful-recon-limited",
                                      "suspicious-filename-detect", "suspicious-login", "system-call-detect",
                                      "unusual-client-port-connection", "web-application-activity", "icmp-event",
                                      "misc-activity", "network-scan", "not-suspicious", "protocol-command-decode",
                                      "string-detect", "unknown", "tcp-connection", "priority_1", "priority_2",
                                      "priority_3", "priority_4", "alerts_weighted_by_priority", "total_alerts",
                                      "severe_alerts", "warning_alerts"]
    OSSEC_IDS_LOG_TOPIC_ATTRIBUTES = ["timestamp", "ip", "total_alerts", "warning_alerts", "severe_alerts",
                                      "alerts_weighted_by_level", "level_0_alerts", "level_1_alerts",
                                      "level_2_alerts", "level_3_alerts", "level_4_alerts", "level_5_alerts",
                                      "level_6_alerts", "level_7_alerts", "level_8_alerts", "level_9_alerts",
                                      "level_10_alerts", "level_11_alerts", "level_12_alerts", "level_13_alerts",
                                      "level_14_alerts", "level_15_alerts", "invalid_login_alerts",
                                      "authentication_success_alerts", "authentication_failed_alerts",
                                      "connection_attempt_alerts", "attacks_alerts", "adduser_alerts", "sshd_alerts",
                                      "ids_alerts", "firewall_alerts", "squid_alerts",
                                      "apache_alerts", "syslog_alerts"]
    SNORT_IDS_RULE_LOG_ATTRIBUTES = ["timestamp", "ip", "alert_rule_id", "num_alerts"]
    SNORT_IDS_IP_LOG_ATTRIBUTES = ["timestamp", "ip", "alert_ip", "attempted-admin", "attempted-user",
                                   "inappropriate-content", "policy-violation", "shellcode-detect",
                                   "successful-admin", "successful-user", "trojan-activity", "unsuccessful-user",
                                   "web-application-attack", "attempted-dos", "attempted-recon", "bad-unknown",
                                   "default-login-attempt", "denial-of-service", "misc-attack",
                                   "non-standard-protocol", "rpc-portmap-decode", "successful-dos",
                                   "successful-recon-largescale", "successful-recon-limited",
                                   "suspicious-filename-detect", "suspicious-login", "system-call-detect",
                                   "unusual-client-port-connection", "web-application-activity", "icmp-event",
                                   "misc-activity", "network-scan", "not-suspicious", "protocol-command-decode",
                                   "string-detect", "unknown", "tcp-connection", "priority_1", "priority_2",
                                   "priority_3", "priority_4", "alerts_weighted_by_priority", "total_alerts",
                                   "severe_alerts", "warning_alerts"]
    HOST_METRICS_TOPIC_ATTRIBUTES = ["timestamp", "ip", "num_logged_in_users", "num_failed_login_attempts",
                                     "num_open_connections", "num_login_events", "num_processes", "num_users"]
    DOCKER_STATS_TOPIC_ATTRIBUTES = ["timestamp", "ip", "cpu_percent", "mem_current", "mem_total",
                                     "mem_percent", "blk_read", "blk_write", "net_rc", "net_tx", "pids"]
    ATTACKER_ACTIONS_ATTRIBUTES = ["timestamp", "id", "description", "index", "name", "time", "ip", "cmd"]
    DEFENDER_ACTIONS_ATTRIBUTES = ["timestamp", "id", "description", "index", "name", "time", "ip", "cmd"]
    OPENFLOW_FLOW_STATS_TOPIC_ATTRIBUTES = ["timestamp", "datapath_id", "in_port", "out_port", "dst_mac_address",
                                            "num_packets", "num_bytes", "duration_nanoseconds",
                                            "duration_seconds", "hard_timeout", "idle_timeout", "priority",
                                            "cookie"]
    OPENFLOW_PORT_STATS_TOPIC_ATTRIBUTES = ["timestamp", "datapath_id", "port", "num_received_packets",
                                            "num_received_bytes", "num_received_errors", "num_transmitted_packets",
                                            "num_transmitted_bytes", "num_transmitted_errors", "num_received_dropped",
                                            "num_transmitted_dropped", "num_received_frame_errors",
                                            "num_received_overrun_errors", "num_received_crc_errors", "num_collisions",
                                            "duration_nanoseconds", "duration_seconds"]
    OPENFLOW_AGG_FLOW_STATS_TOPIC_ATTRIBUTES = ["timestamp", "datapath_id", "total_num_packets", "total_num_bytes",
                                                "total_num_flows"]
    AVERAGE_OPENFLOW_FLOW_STATS_PER_SWITCH_TOPIC_ATTRIBUTES = [
        "timestamp", "datapath_id", "total_num_packets", "total_num_bytes", "avg_duration_nanoseconds",
        "avg_duration_seconds", "avg_hard_timeout", "avg_idle_timeout", "avg_priority", "avg_cookie"]
    AVERAGE_OPENFLOW_PORT_STATS_PER_SWITCH_TOPIC_ATTRIBUTES = [
        "timestamp", "datapath_id", "total_num_received_packets", "total_num_received_bytes",
        "total_num_received_errors", "total_num_transmitted_packets", "total_num_transmitted_bytes",
        "total_num_transmitted_errors", "total_num_received_dropped", "total_num_transmitted_dropped",
        "total_num_received_frame_errors", "total_num_received_overrun_errors",
        "total_num_received_crc_errors", "total_num_collisions", "avg_duration_nanoseconds", "avg_duration_seconds"]

    SNORT_IDS_ALERTS_LABELS = [
        "total_alerts", "warning_alerts", "severe_alerts", "alerts_weighted_by_priority",
        "priority_1_alerts", "priority_2_alerts",
        "priority_3_alerts", "priority_4_alerts", "attempted-admin_alerts",
        "attempted-user_alerts", "inappropriate-content_alerts", "policy-violation_alerts",
        "shellcode-detect_alerts", "successful-admin_alerts",
        "successful-user_alerts", "trojan-activity_alerts", "unsuccessful-user_alerts",
        "web-application-attack_alerts",
        "attempted-dos_alerts", "attempted-recon_alerts", "bad-unknown_alerts",
        "default-login-attempt_alerts",
        "denial-of-service_alerts", "misc-attack_alerts", "non-standard-protocol_alerts",
        "rpc-portmap-decode_alerts",
        "successful-dos_alerts", "successful-recon-largescale_alerts", "successful-recon-limited_alerts",
        "suspicious-filename-detect_alerts", "suspicious-login_alerts", "system-call-detect_alerts",
        "unusual-client-port-connection_alerts", "web-application-activity_alerts", "icmp-event_alerts",
        "misc-activity_alerts", "network-scan_alerts", "not-suspicious_alerts", "protocol-command-decode_alerts",
        "string-detect_alerts", "unknown_alerts", "tcp-connection_alerts"
    ]
    OSSEC_IDS_ALERTS_LABELS = [
        "total_alerts", "warning_alerts", "severe_alerts", "alerts_weighted_by_level",
        "level_0_alerts", "level_1_alerts",
        "level_2_alerts", "level_3_alerts", "level_4_alerts", "level_5_alerts", "level_6_alerts", "level_7_alerts",
        "level_8_alerts", "level_9_alerts", "level_10_alerts", "level_11_alerts", "level_12_alerts",
        "level_13_alerts", "level_14_alerts", "level_15_alerts",
        "invalid_login_alerts", "authentication_success_alerts", "authentication_failed_alerts",
        "connection_attempt_alerts", "attacks_alerts", "adduser_alerts", "sshd_alerts", "ids_alerts",
        "firewall_alerts", "squid_alerts", "apache_alerts", "syslog_alerts"
    ]
    HOST_METRICS_LABELS = [
        "num_logged_in_users", "severe_alerts", "warning_alerts",
        "num_failed_login_attempts", "num_open_connections", "num_login_events",
        "num_processes", "num_users"
    ]
    DOCKER_STATS_COUNTER_LABELS = [
        "pids", "cpu_percent", "mem_current", "mem_total",
        "mem_percent", "blk_read", "blk_write", "net_rx", "net_tx"
    ]
    DOCKER_STATS_PERCENT_LABELS = [
        "cpu_percent", "cpu_percent"
    ]
    CLIENT_POPULATION_METRIC_LABELS = ["num_clients", "rate", "service_time"]
    ALL_DELTA_AGG_LABELS = (SNORT_IDS_ALERTS_LABELS + HOST_METRICS_LABELS + DOCKER_STATS_COUNTER_LABELS +
                            DOCKER_STATS_PERCENT_LABELS + CLIENT_POPULATION_METRIC_LABELS + OSSEC_IDS_ALERTS_LABELS)
    ALL_DELTA_MACHINE_LABELS = (SNORT_IDS_ALERTS_LABELS + HOST_METRICS_LABELS + DOCKER_STATS_COUNTER_LABELS +
                                DOCKER_STATS_PERCENT_LABELS + OSSEC_IDS_ALERTS_LABELS)
    ALL_INITIAL_AGG_LABELS = (HOST_METRICS_LABELS + DOCKER_STATS_COUNTER_LABELS + DOCKER_STATS_PERCENT_LABELS +
                              CLIENT_POPULATION_METRIC_LABELS + SNORT_IDS_ALERTS_LABELS + OSSEC_IDS_ALERTS_LABELS)
    ALL_INITIAL_MACHINE_LABELS = (HOST_METRICS_LABELS + DOCKER_STATS_COUNTER_LABELS + DOCKER_STATS_PERCENT_LABELS +
                                  SNORT_IDS_ALERTS_LABELS + OSSEC_IDS_ALERTS_LABELS)


class KAFKA:
    """
    String constants for managing Kafka
    """
    KAFKA_STATUS = "service kafka status"
    KAFKA_STOP = "service kafka stop"
    KAFKA_START = "service kafka start"
    RETENTION_MS_CONFIG_PROPERTY = "retention.ms"
    BOOTSTRAP_SERVERS_PROPERTY = "bootstrap.servers"
    CLIENT_ID_PROPERTY = "client.id"
    GROUP_ID_PROPERTY = "group.id"
    AUTO_OFFSET_RESET_PROPERTY = "auto.offset.reset"
    EARLIEST_OFFSET = "earliest"
    PORT = 9092
    EXTERNAL_PORT = 9292
    INTERNAL_IP_PLACEHOLDER = "INTERNAL_IP"
    EXTERNAL_IP_PLACEHOLDER = "EXTERNAL_IP"
    DIR = "/usr/local/kafka/logs/"
    KAFKA_CONFIG_FILE = "/usr/local/kafka/config/server.properties"


class ELK:
    """
    String constants for managing the ELK stack
    """
    ELK_START = "nohup /usr/local/bin/start.sh > /elk_server.log &"
    ELK_LOG = "/elk_server.log"
    ELASTICSEARCH_STOP = "service elasticsearch stop"
    KIBANA_STOP = "service kibana stop"
    LOGSTASH_STOP = "service logstash stop"
    ELASTICSEARCH_START = "service elasticsearch start"
    KIBANA_START = "service kibana start"
    LOGSTASH_START = "service logstash start"
    ELASTICSEARCH_STATUS = "service elasticsearch status"
    KIBANA_STATUS = "service kibana status"
    LOGSTASH_STATUS = "service logstash status"
    ELASTICSEARCH_LOG_DIR = "/var/log/elasticsearch/"
    LOGSTASH_LOG_DIR = "/var/log/logstash/"
    KIBANA_LOG_DIR = "/var/log/kibana/"
    ELASTIC_PORT = 9200
    KIBANA_PORT = 5601
    LOGSTASH_PORT = 5044


class RYU:
    """
    String constants for managing Ryu
    """
    CHECK_IF_RYU_CONTROLLER_IS_RUNNING = "ps -aux | grep ryu_controller.py"
    STOP_RYU_CONTROLLER = "sudo pkill -f ryu_controller.py"
    STOP_RYU_CONTROLLER_MANAGER = "sudo pkill -f ryu-manager"
    RYU_CONTROLLER_FILENAME = "ryu_controller.py"
    SEARCH_CONTROLLER = "/root/miniconda3/bin/python3 /ryu_controller.py"
    START_RYU_CONTROLLER = "sudo nohup /root/miniconda3/bin/python3 /ryu_controller.py --port {} --webport {} " \
                           "--controller {} &"
    START_PRODUCER_HTTP_RESOURCE = "/cslenorthboundapi/producer/start"
    STOP_PRODUCER_HTTP_RESOURCE = "/cslenorthboundapi/producer/stop"
    STATUS_PRODUCER_HTTP_RESOURCE = "/cslenorthboundapi/producer/status"
    TIME_STEP_LEN_SECONDS = "time_step_len_seconds"
    PRODUCER_RUNNING = "producer_running"
    KAFKA_CONF = "kafka_conf"
    REQUEST_TIMEOUT_S = 5


class INTERFACES:
    """
    String constrants related to networking interfaces
    """
    ETH1 = "eth1"
    ETH0 = "eth0"
    ADDR = "addr"


class SPARK:
    """
    Constants related to Spark
    """
    START_SPARK_MASTER = "/spark-3.3.2-bin-hadoop3/sbin/start-master.sh"
    START_SPARK_WORKER = "/spark-3.3.2-bin-hadoop3/sbin/start-worker.sh spark://15.13.1.161:7077 -m 2G -c 1"
    SPARK_MASTER_PID_FILE = "/tmp/spark--org.apache.spark.deploy.master.Master-1.pid"
    SPARK_WORKER_PID_FILE = "/tmp/spark--org.apache.spark.deploy.worker.Worker-1.pid"
    STOP_SPARK_WORKER = "sudo kill -9 {}"
    STOP_SPARK_MASTER = "sudo kill -9 {}"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: snort_ids_manager.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17snort_ids_manager.proto\"@\n\x14GetSnortIdsAlertsMsg\x12\x11\n\ttimestamp\x18\x01 \x01(\x02\x12\x15\n\rlog_file_path\x18\x02 \x01(\t\"\x18\n\x16StopSnortIdsMonitorMsg\"u\n\x17StartSnortIdsMonitorMsg\x12\x10\n\x08kafka_ip\x18\x01 \x01(\t\x12\x12\n\nkafka_port\x18\x02 \x01(\x05\x12\x15\n\rlog_file_path\x18\x03 \x01(\t\x12\x1d\n\x15time_step_len_seconds\x18\x04 \x01(\x05\"\x11\n\x0fStopSnortIdsMsg\"[\n\x10StartSnortIdsMsg\x12\x19\n\x11ingress_interface\x18\x01 \x01(\t\x12\x18\n\x10\x65gress_interface\x18\x02 \x01(\t\x12\x12\n\nsubnetmask\x18\x03 \x01(\t\"\x1d\n\x1bGetSnortIdsMonitorStatusMsg\"H\n\x12SnortIdsMonitorDTO\x12\x17\n\x0fmonitor_running\x18\x01 \x01(\x08\x12\x19\n\x11snort_ids_running\x18\x02 \x01(\x08\"\x8a\x0b\n\x0eSnortIdsLogDTO\x12\x11\n\ttimestamp\x18\x01 \x01(\x02\x12\n\n\x02ip\x18\x02 \x01(\t\x12\x1e\n\x16\x61ttempted_admin_alerts\x18\x03 \x01(\x05\x12\x1d\n\x15\x61ttempted_user_alerts\x18\x04 \x01(\x05\x12$\n\x1cinappropriate_content_alerts\x18\x05 \x01(\x05\x12\x1f\n\x17policy_violation_alerts\x18\x06 \x01(\x05\x12\x1f\n\x17shellcode_detect_alerts\x18\x07 \x01(\x05\x12\x1f\n\x17successful_admin_alerts\x18\x08 \x01(\x05\x12\x1e\n\x16successful_user_alerts\x18\t \x01(\x05\x12\x1e\n\x16trojan_activity_alerts\x18\n \x01(\x05\x12 \n\x18unsuccessful_user_alerts\x18\x0b \x01(\x05\x12%\n\x1dweb_application_attack_alerts\x18\x0c \x01(\x05\x12\x1c\n\x14\x61ttempted_dos_alerts\x18\r \x01(\x05\x12\x1e\n\x16\x61ttempted_recon_alerts\x18\x0e \x01(\x05\x12\x1a\n\x12\x62\x61\x64_unknown_alerts\x18\x0f \x01(\x05\x12$\n\x1c\x64\x65\x66\x61ult_login_attempt_alerts\x18\x10 \x01(\x05\x12 \n\x18\x64\x65nial_of_service_alerts\x18\x11 \x01(\x05\x12\x1a\n\x12misc_attack_alerts\x18\x12 \x01(\x05\x12$\n\x1cnon_standard_protocol_alerts\x18\x13 \x01(\x05\x12!\n\x19rpc_portman_decode_alerts\x18\x14 \x01(\x05\x12\x1d\n\x15successful_dos_alerts\x18\x15 \x01(\x05\x12*\n\"successful_recon_largescale_alerts\x18\x16 \x01(\x05\x12\'\n\x1fsuccessful_recon_limited_alerts\x18\x17 \x01(\x05\x12)\n!suspicious_filename_detect_alerts\x18\x18 \x01(\x05\x12\x1f\n\x17suspicious_login_alerts\x18\x19 \x01(\x05\x12!\n\x19system_call_detect_alerts\x18\x1a \x01(\x05\x12-\n%unusual_client_port_connection_alerts\x18\x1b \x01(\x05\x12\'\n\x1fweb_application_activity_alerts\x18\x1c \x01(\x05\x12\x19\n\x11icmp_event_alerts\x18\x1d \x01(\x05\x12\x1c\n\x14misc_activity_alerts\x18\x1e \x01(\x05\x12\x1b\n\x13network_scan_alerts\x18\x1f \x01(\x05\x12\x1d\n\x15not_suspicious_alerts\x18  \x01(\x05\x12&\n\x1eprotocol_command_decode_alerts\x18! \x01(\x05\x12\x1c\n\x14string_detect_alerts\x18\" \x01(\x05\x12\x16\n\x0eunknown_alerts\x18# \x01(\x05\x12\x1d\n\x15tcp_connection_alerts\x18$ \x01(\x05\x12\x19\n\x11priority_1_alerts\x18% \x01(\x05\x12\x19\n\x11priority_2_alerts\x18& \x01(\x05\x12\x19\n\x11priority_3_alerts\x18\' \x01(\x05\x12\x19\n\x11priority_4_alerts\x18( \x01(\x05\x12\x14\n\x0ctotal_alerts\x18) \x01(\x05\x12\x16\n\x0ewarning_alerts\x18* \x01(\x05\x12\x15\n\rsevere_alerts\x18+ \x01(\x05\x12#\n\x1b\x61lerts_weighted_by_priority\x18, \x01(\x05\x32\xa5\x03\n\x0fSnortIdsManager\x12=\n\x11getSnortIdsAlerts\x12\x15.GetSnortIdsAlertsMsg\x1a\x0f.SnortIdsLogDTO\"\x00\x12\x45\n\x13stopSnortIdsMonitor\x12\x17.StopSnortIdsMonitorMsg\x1a\x13.SnortIdsMonitorDTO\"\x00\x12G\n\x14startSnortIdsMonitor\x12\x18.StartSnortIdsMonitorMsg\x1a\x13.SnortIdsMonitorDTO\"\x00\x12O\n\x18getSnortIdsMonitorStatus\x12\x1c.GetSnortIdsMonitorStatusMsg\x1a\x13.SnortIdsMonitorDTO\"\x00\x12\x37\n\x0cstopSnortIds\x12\x10.StopSnortIdsMsg\x1a\x13.SnortIdsMonitorDTO\"\x00\x12\x39\n\rstartSnortIds\x12\x11.StartSnortIdsMsg\x1a\x13.SnortIdsMonitorDTO\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'snort_ids_manager_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _GETSNORTIDSALERTSMSG._serialized_start=27
  _GETSNORTIDSALERTSMSG._serialized_end=91
  _STOPSNORTIDSMONITORMSG._serialized_start=93
  _STOPSNORTIDSMONITORMSG._serialized_end=117
  _STARTSNORTIDSMONITORMSG._serialized_start=119
  _STARTSNORTIDSMONITORMSG._serialized_end=236
  _STOPSNORTIDSMSG._serialized_start=238
  _STOPSNORTIDSMSG._serialized_end=255
  _STARTSNORTIDSMSG._serialized_start=257
  _STARTSNORTIDSMSG._serialized_end=348
  _GETSNORTIDSMONITORSTATUSMSG._serialized_start=350
  _GETSNORTIDSMONITORSTATUSMSG._serialized_end=379
  _SNORTIDSMONITORDTO._serialized_start=381
  _SNORTIDSMONITORDTO._serialized_end=453
  _SNORTIDSLOGDTO._serialized_start=456
  _SNORTIDSLOGDTO._serialized_end=1874
  _SNORTIDSMANAGER._serialized_start=1877
  _SNORTIDSMANAGER._serialized_end=2298
# @@protoc_insertion_point(module_scope)



import re
import datetime
import csle_collector.constants.constants as constants


class SnortIdsFastLogAlert:
    """
    DTO representing an alert entry in the fast log of Snort
    """

    def __init__(self, timestamp: float, priority: int, class_id: int, source_ip: str, target_ip: str, rule_id: str) \
            -> None:
        """
        Initializes the DTO

        :param timestamp: the timestamp of the record
        :param priority: the priority of the record
        :param class_id: the class id of the record
        :param source_ip: the source ip of the record
        :param target_ip: the target ip of the record
        :param rule_id: the id of the Snort rule relating to the record
        """
        self.timestamp = timestamp
        self.priority = priority
        self.class_id = class_id
        self.source_ip = source_ip
        self.target_ip = target_ip
        self.rule_id = rule_id


class SnortIdsAlert:
    """
    Object representing an IDS Alert
    """

    def __init__(self):
        """
        Initializes the IDS Alert Fields
        """
        self.timestamp = None
        self.sig_generator = None
        self.sig_id = None
        self.sig_rev = None
        self.msg = None
        self.proto = None
        self.src_ip = None
        self.src_port = None
        self.dst_ip = None
        self.dst_port = None
        self.eth_src = None
        self.eth_dst = None
        self.eth_len = None
        self.tcp_flags = None
        self.tcp_seq = None
        self.tcp_ack = None
        self.tcp_len = None
        self.tcp_window = None
        self.ttl = None
        self.tos = None
        self.id = None
        self.dgm_len = None
        self.ip_len = None
        self.icmp_type = None
        self.icmp_code = None
        self.icmp_id = None
        self.icmp_seq = None
        self.priority = None

    @staticmethod
    def parse_from_str(csv_str_record: str, year: int) -> "SnortIdsAlert":
        """
        Parses the IDS alert from a string

        :param csv_str_record: the string to parse
        :param year: the year of the entry
        :return: the parsed IDS Alert
        """
        a_fields = csv_str_record.split(",")
        alert_dao = SnortIdsAlert()
        if len(a_fields) > 1:
            alert_dao.timestamp = a_fields[0]
            if alert_dao.timestamp is not None and alert_dao.timestamp != "" and alert_dao.timestamp != "0":
                alert_dao.timestamp = str(year) + " " + alert_dao.timestamp
                try:
                    alert_dao.timestamp = datetime.datetime.strptime(alert_dao.timestamp.strip(),
                                                                     '%Y %m/%d-%H:%M:%S.%f').timestamp()
                except Exception:
                    alert_dao.timestamp = datetime.datetime.strptime("2010 04/20-08:46:14.094913",
                                                                     '%Y %m/%d-%H:%M:%S.%f').timestamp()
            else:
                alert_dao.timestamp = datetime.datetime.strptime("2010 04/20-08:46:14.094913",
                                                                 '%Y %m/%d-%H:%M:%S.%f').timestamp()
        else:
            alert_dao.timestamp = datetime.datetime.strptime("2010 04/20-08:46:14.094913",
                                                             '%Y %m/%d-%H:%M:%S.%f').timestamp()
        if len(a_fields) > 1:
            alert_dao.sig_generator = a_fields[1]
        if len(a_fields) > 2:
            alert_dao.sig_id = a_fields[2]
        if len(a_fields) > 3:
            alert_dao.sig_rev = a_fields[3]
        if len(a_fields) > 4:
            alert_dao.msg = a_fields[4]
        if len(a_fields) > 5:
            alert_dao.proto = a_fields[5]
        if len(a_fields) > 6:
            alert_dao.src_ip = a_fields[6]
        if len(a_fields) > 7:
            alert_dao.src_port = a_fields[7]
        if len(a_fields) > 8:
            alert_dao.dst_ip = a_fields[8]
        if len(a_fields) > 9:
            alert_dao.dst_port = a_fields[9]
        if len(a_fields) > 10:
            alert_dao.eth_src = a_fields[10]
        if len(a_fields) > 11:
            alert_dao.eth_dst = a_fields[11]
        if len(a_fields) > 12:
            alert_dao.eth_len = a_fields[12]
        if len(a_fields) > 13:
            alert_dao.tcp_flags = a_fields[13]
        if len(a_fields) > 14:
            alert_dao.tcp_seq = a_fields[14]
        if len(a_fields) > 15:
            alert_dao.tcp_ack = a_fields[15]
        if len(a_fields) > 16:
            alert_dao.tcp_len = a_fields[16]
        if len(a_fields) > 17:
            alert_dao.tcp_window = a_fields[17]
        if len(a_fields) > 18:
            alert_dao.ttl = a_fields[18]
        if len(a_fields) > 19:
            alert_dao.tos = a_fields[19]
        if len(a_fields) > 20:
            alert_dao.id = a_fields[20]
        if len(a_fields) > 21:
            alert_dao.dgm_len = a_fields[21]
        if len(a_fields) > 22:
            alert_dao.ip_len = a_fields[22]
        if len(a_fields) > 23:
            alert_dao.icmp_type = a_fields[23]
        if len(a_fields) > 24:
            alert_dao.icmp_code = a_fields[24]
        if len(a_fields) > 25:
            alert_dao.icmp_id = a_fields[25]
        if len(a_fields) > 26:
            alert_dao.icmp_seq = a_fields[26]
        alert_dao.priority = 1
        return alert_dao

    def set_priority(self, priority: int) -> None:
        """
        Sets the priority of the alert DTO

        :param priority: the priority to set
        :return: None
        """
        self.priority = priority

    @staticmethod
    def fast_log_parse(fast_log_str: str, year: int) -> SnortIdsFastLogAlert:
        """
        Parses the IDS Alert from a given string from the fast-log of Snort

        :param fast_log_str: the fast log string to parse
        :param year: the year
        :return: the priority, the class, and the time-stamp
        """
        priorities = re.findall(constants.SNORT_IDS_ROUTER.PRIORITY_REGEX, fast_log_str)
        if len(priorities) > 0:
            temp = priorities[0].replace("Priority: ", "")
            priority = int(temp)
        else:
            priority = 1
        alert_classes = re.findall(constants.SNORT_IDS_ROUTER.CLASSIFICATION_REGEX, fast_log_str)
        alert_class = "unknown"
        alert_class_id = 1
        if len(alert_classes) > 0:
            alert_class = alert_classes[0]
        if alert_class in constants.SNORT_IDS_ROUTER.SNORT_ALERT_IDS_ID:
            alert_class_id = constants.SNORT_IDS_ROUTER.SNORT_ALERT_IDS_ID[alert_class]

        ts_str = fast_log_str.split(" ")[0]
        if ts_str is not None and ts_str != "":
            ts_str = ts_str.strip()
            if ts_str != "":
                ts_str = str(year) + " " + ts_str
                try:
                    ts = datetime.datetime.strptime(ts_str.strip(), '%Y %m/%d-%H:%M:%S.%f').timestamp()
                except Exception:
                    ts = datetime.datetime.strptime("2010 04/20-08:46:14.094913", '%Y %m/%d-%H:%M:%S.%f').timestamp()
            else:
                ts = datetime.datetime.strptime("2010 04/20-08:46:14.094913", '%Y %m/%d-%H:%M:%S.%f').timestamp()

        source_ip = ""
        target_ip = ""
        ips_match = re.findall(constants.SNORT_IDS_ROUTER.IPS_REGEX, fast_log_str)
        if len(ips_match) > 0:
            ips = ips_match[0].replace(" ", "").split("->")
            source_ip = ips[0]
            target_ip = ips[1]

        rule_id = ""
        rule_match = re.findall(constants.SNORT_IDS_ROUTER.RULE_ID_REGEX, fast_log_str)
        if len(rule_match) > 0:
            rule_id = rule_match[0].replace("[", "").replace(":", "-")

        fast_log_alert = SnortIdsFastLogAlert(timestamp=ts, priority=priority, class_id=alert_class_id,
                                              source_ip=source_ip, target_ip=target_ip, rule_id=rule_id)
        return fast_log_alert



from typing import List, Dict, Any
import random
import numpy as np
import csle_collector.client_manager.client_manager_pb2
from csle_base.json_serializable import JSONSerializable
from csle_base.grpc_serializable import GRPCSerializable


class WorkflowMarkovChain(JSONSerializable, GRPCSerializable):
    """
    A Markov chain representation of a workflow.
    """

    def __init__(self, transition_matrix: List[List[float]], initial_state: int, id: int) -> None:
        """
        Initializes the object

        :param transition_matrix: the transition matrix of the workflow Markov chain (row-stochastic)
        :param initial_state: the initial state of the Markov chain
        :param id: the id of the workflow
        """
        self.transition_matrix = transition_matrix
        for i in range(len(transition_matrix)):
            assert round(sum(self.transition_matrix[i]), 3) == 1
        self.initial_state = initial_state
        self.current_state = initial_state
        self.t = 0
        self.id = id

    def step_forward(self) -> int:
        """
        Move t forward by one and possibly transition to a new state.
        Returns the new state.
        """
        self.t += 1
        next_state = WorkflowMarkovChain.markov_next_state(row=self.transition_matrix[self.current_state])
        self.current_state = next_state
        return self.current_state

    def reset(self) -> None:
        """
        Resets the Markov chain

        :return: None
        """
        self.current_state = self.initial_state
        self.t = 0

    @staticmethod
    def markov_next_state(row: List[float]) -> int:
        """
        Samples the next state of the Markov chain

        :param row: a numpy array of probabilities that sum up to 1.
        :return:  an integer from 0 to len(row)-1.
        """
        cumsum = np.cumsum(row)  # cumulative sum of probabilities
        r = random.random()  # random number between 0 and 1
        for i in range(len(cumsum)):
            if r < cumsum[i]:  # find the first index where r is smaller than cumsum[i]
                return i  # return that index as the outcome
        raise ValueError("Invalid transition probabilities")

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "WorkflowMarkovChain":
        """
        Converts a dict representation to an instance

        :param d: the dict to convert
        :return: the created instance
        """
        obj = WorkflowMarkovChain(transition_matrix=d["transition_matrix"], initial_state=d["initial_state"],
                                  id=d["id"])
        obj.current_state = d["current_state"]
        obj.t = d["t"]
        return obj

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation

        :return: a dict representation of the object
        """
        d: Dict[str, Any] = {}
        d["transition_matrix"] = self.transition_matrix
        d["initial_state"] = self.initial_state
        d["current_state"] = self.current_state
        d["t"] = self.t
        d["id"] = self.id
        return d

    @staticmethod
    def from_json_file(json_file_path: str) -> "WorkflowMarkovChain":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return WorkflowMarkovChain.from_dict(json.loads(json_str))

    def copy(self) -> "WorkflowMarkovChain":
        """
        :return: a copy of the DTO
        """
        return WorkflowMarkovChain.from_dict(self.to_dict())

    def to_grpc_object(self) -> csle_collector.client_manager.client_manager_pb2.WorkflowMarkovChainDTO:
        """
        :return: a GRPC serializable version of the object
        """
        rows = []
        for i in range(len(self.transition_matrix)):
            rows.append(csle_collector.client_manager.client_manager_pb2.ProbabilityDistributionDTO(
                probabilities=self.transition_matrix[i]))
        transition_matrix = csle_collector.client_manager.client_manager_pb2.TransitionMatrixDTO(rows=rows)
        return csle_collector.client_manager.client_manager_pb2.WorkflowMarkovChainDTO(
            initial_state=self.initial_state, id=self.id, transition_matrix=transition_matrix)

    @staticmethod
    def from_grpc_object(obj: csle_collector.client_manager.client_manager_pb2.WorkflowMarkovChainDTO) \
            -> "WorkflowMarkovChain":
        """
        Instantiates the object from a GRPC DTO

        :param obj: the object to instantiate from
        :return: the instantiated object
        """
        transition_matrix = []
        for i in range(len(obj.transition_matrix.rows)):
            transition_matrix.append(list(obj.transition_matrix.rows[i].probabilities))
        return WorkflowMarkovChain(id=obj.id, initial_state=obj.initial_state, transition_matrix=transition_matrix)

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return f"Workflow MC, id: {self.id}, initial_state: {self.initial_state}, " \
               f"transition_matrix: {self.transition_matrix}"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: docker_stats_manager.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x64ocker_stats_manager.proto\"P\n\x19StopDockerStatsMonitorMsg\x12\x11\n\temulation\x18\x01 \x01(\t\x12 \n\x18\x65xecution_first_ip_octet\x18\x02 \x01(\x05\",\n\x0b\x43ontainerIp\x12\n\n\x02ip\x18\x01 \x01(\t\x12\x11\n\tcontainer\x18\x02 \x01(\t\"\xd5\x01\n\x1aStartDockerStatsMonitorMsg\x12\x11\n\temulation\x18\x01 \x01(\t\x12 \n\x18\x65xecution_first_ip_octet\x18\x02 \x01(\x05\x12\x10\n\x08kafka_ip\x18\x03 \x01(\t\x12\x1b\n\x13stats_queue_maxsize\x18\x04 \x01(\x05\x12\x1d\n\x15time_step_len_seconds\x18\x05 \x01(\x05\x12\x12\n\nkafka_port\x18\x06 \x01(\x05\x12 \n\ncontainers\x18\x07 \x03(\x0b\x32\x0c.ContainerIp\" \n\x1eGetDockerStatsMonitorStatusMsg\"_\n\x15\x44ockerStatsMonitorDTO\x12\x14\n\x0cnum_monitors\x18\x01 \x01(\x05\x12\x12\n\nemulations\x18\x02 \x03(\t\x12\x1c\n\x14\x65mulation_executions\x18\x03 \x03(\x05\x32\x90\x02\n\x12\x44ockerStatsManager\x12X\n\x1bgetDockerStatsMonitorStatus\x12\x1f.GetDockerStatsMonitorStatusMsg\x1a\x16.DockerStatsMonitorDTO\"\x00\x12N\n\x16stopDockerStatsMonitor\x12\x1a.StopDockerStatsMonitorMsg\x1a\x16.DockerStatsMonitorDTO\"\x00\x12P\n\x17startDockerStatsMonitor\x12\x1b.StartDockerStatsMonitorMsg\x1a\x16.DockerStatsMonitorDTO\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'docker_stats_manager_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STOPDOCKERSTATSMONITORMSG._serialized_start=30
  _STOPDOCKERSTATSMONITORMSG._serialized_end=110
  _CONTAINERIP._serialized_start=112
  _CONTAINERIP._serialized_end=156
  _STARTDOCKERSTATSMONITORMSG._serialized_start=159
  _STARTDOCKERSTATSMONITORMSG._serialized_end=372
  _GETDOCKERSTATSMONITORSTATUSMSG._serialized_start=374
  _GETDOCKERSTATSMONITORSTATUSMSG._serialized_end=406
  _DOCKERSTATSMONITORDTO._serialized_start=408
  _DOCKERSTATSMONITORDTO._serialized_end=503
  _DOCKERSTATSMANAGER._serialized_start=506
  _DOCKERSTATSMANAGER._serialized_end=778
# @@protoc_insertion_point(module_scope)



from typing import List
import csle_collector.traffic_manager.traffic_manager_pb2_grpc
import csle_collector.traffic_manager.traffic_manager_pb2
import csle_collector.constants.constants as constants


def get_traffic_status(stub: csle_collector.traffic_manager.traffic_manager_pb2_grpc.TrafficManagerStub,
                       timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO:
    """
    Queries the server for the Traffic generator status

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a TrafficDTO describing the status of the traffic generator
    """
    get_traffic_status_msg = csle_collector.traffic_manager.traffic_manager_pb2.GetTrafficStatusMsg()
    traffic_dto: csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO = \
        stub.getTrafficStatus(get_traffic_status_msg, timeout=timeout)
    return traffic_dto


def stop_traffic(stub: csle_collector.traffic_manager.traffic_manager_pb2_grpc.TrafficManagerStub,
                 timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO:
    """
    Sends a request to the traffic manager to stop the traffic generator

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a TrafficDTO describing the status of the traffic generator
    """
    stop_traffic_msg = csle_collector.traffic_manager.traffic_manager_pb2.StopTrafficMsg()
    traffic_dto: csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO = \
        stub.stopTraffic(stop_traffic_msg, timeout=timeout)
    return traffic_dto


def start_traffic(stub: csle_collector.traffic_manager.traffic_manager_pb2_grpc.TrafficManagerStub,
                  commands: List[str], sleep_time: int,
                  timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO:
    """
    Sends a request to the traffic manager to start the traffic generator script

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :param commands: list of commands for the traffic generator
    :param sleep_time: sleep time for the traffic generator
    :return: an TrafficDTO describing the status of the traffic manager
    """
    start_traffic_msg = csle_collector.traffic_manager.traffic_manager_pb2.StartTrafficMsg(
        commands=commands, sleepTime=sleep_time)
    traffic_dto: csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO = \
        stub.startTraffic(start_traffic_msg, timeout=timeout)
    return traffic_dto



from typing import Union
import logging
import socket
import grpc
import subprocess
import netifaces
from concurrent import futures
import csle_collector.constants.constants as constants
import csle_collector.ossec_ids_manager.ossec_ids_manager_pb2_grpc
import csle_collector.ossec_ids_manager.ossec_ids_manager_pb2
from csle_collector.ossec_ids_manager.ossec_ids_manager_util import OSSecManagerUtil
from csle_collector.ossec_ids_manager.threads.ossec_ids_monitor_thread import OSSecIdsMonitorThread


class OSSECIdsManagerServicer(csle_collector.ossec_ids_manager.ossec_ids_manager_pb2_grpc.OSSECIdsManagerServicer):
    """
    gRPC server for collecting OSSEC IDS statistics.
    """

    def __init__(self) -> None:
        """
        Initializes the server
        """
        logging.basicConfig(filename=f"{constants.LOG_FILES.OSSEC_IDS_MANAGER_LOG_DIR}"
                                     f"{constants.LOG_FILES.OSSEC_IDS_MANAGER_LOG_FILE}", level=logging.INFO)
        self.hostname = socket.gethostname()
        try:
            self.ip = netifaces.ifaddresses(constants.INTERFACES.ETH0)[netifaces.AF_INET][0][constants.INTERFACES.ADDR]
        except Exception:
            self.ip = socket.gethostbyname(self.hostname)
        self.conf = {
            constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: f"{self.ip}:{constants.KAFKA.PORT}",
            constants.KAFKA.CLIENT_ID_PROPERTY: self.hostname}
        self.ids_monitor_thread: Union[None, OSSecIdsMonitorThread] = None
        logging.info(f"Starting the OSSEC IDSManager hostname: {self.hostname} ip: {self.ip}")

    def _is_ossec_running(self) -> bool:
        """
        Utility method to check if OSSEC is running

        :return: status and list of topics
        """
        logging.info("Checking if OSSEC IDS is running")
        status_output = subprocess.run(constants.OSSEC.CHECK_IF_OSSEC_IS_RUNNING_CMD.split(" "),
                                       capture_output=True, text=True).stdout
        logging.info(f"Status output: {status_output}")
        running = constants.OSSEC.OSSEC_RUNNING_SEARCH in status_output
        logging.info(f"Running status: {running}")
        return running

    def _is_monitor_running(self) -> bool:
        """
        Utility method to check if the monitor is running

        :return: True if running else false
        """
        if self.ids_monitor_thread is not None:
            return self.ids_monitor_thread.running
        return False

    def getOSSECIdsAlerts(self, request: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.GetOSSECIdsAlertsMsg,
                          context: grpc.ServicerContext) \
            -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsLogDTO:
        """
        Gets the statistics of the OSSEC IDS log from a given timestamp

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with IDS statistics
        """
        alert_counters = OSSecManagerUtil.read_ossec_ids_data(request.timestamp)
        ossec_ids_log_dto = alert_counters.to_grpc_object()
        ossec_ids_log_dto.ip = self.ip
        return ossec_ids_log_dto

    def startOSSECIdsMonitor(self,
                             request: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.StartOSSECIdsMonitorMsg,
                             context: grpc.ServicerContext) \
            -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO:
        """
        Starts the OSSEC IDS monitor thread

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the IDS monitor thread
        """
        logging.info(f"Starting the OSSEC IDSMonitor thread, timestep length: {request.time_step_len_seconds}, "
                     f"log file path: {request.log_file_path}, kafka ip: {request.kafka_ip}, "
                     f"kafka port: {request.kafka_port}")
        if self.ids_monitor_thread is not None:
            self.ids_monitor_thread.running = False
        self.ids_monitor_thread = OSSecIdsMonitorThread(kafka_ip=request.kafka_ip, kafka_port=request.kafka_port,
                                                        ip=self.ip, hostname=self.hostname,
                                                        log_file_path=request.log_file_path,
                                                        time_step_len_seconds=request.time_step_len_seconds)
        self.ids_monitor_thread.start()
        logging.info("Started the OSSEC IDSMonitor thread")
        ossec_ids_running = self._is_ossec_running()
        return csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO(
            monitor_running=True, ossec_ids_running=ossec_ids_running)

    def stopOSSECIdsMonitor(self,
                            request: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.StartOSSECIdsMonitorMsg,
                            context: grpc.ServicerContext) \
            -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO:
        """
        Stops the OSSEC IDS monitor thread if it is running

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the IDS monitor thread
        """
        if self.ids_monitor_thread is not None:
            self.ids_monitor_thread.running = False
        ossec_ids_running = self._is_ossec_running()
        return csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO(
            monitor_running=False, ossec_ids_running=ossec_ids_running)

    def startOSSECIds(self, request: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.StartOSSECIdsMsg,
                      context: grpc.ServicerContext) \
            -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO:
        """
        Starts the OSSEC IDS

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the IDS and its monitor thread
        """
        logging.info("Starting the OSSEC IDS")
        monitor_running = self._is_monitor_running()
        ossec_running = self._is_ossec_running()
        if ossec_running:
            result = subprocess.run(constants.OSSEC.STOP_OSSEC_IDS.split(" "), capture_output=True, text=True)
            logging.info(f"Stopped the OSSEC IDS, stdout: {result.stdout}, stderr: {result.stderr}")
        if not ossec_running:
            result = subprocess.run(constants.OSSEC.START_OSSEC_IDS.split(" "), capture_output=True, text=True)
            logging.info(f"Started the OSSEC IDS, stdout: {result.stdout}, stderr: {result.stderr}")
        logging.info("Started the OSSEC IDS")
        return csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO(
            monitor_running=monitor_running, ossec_ids_running=True)

    def stopOSSECIds(self, request: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.StartOSSECIdsMsg,
                     context: grpc.ServicerContext) \
            -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO:
        """
        Stops the OSSEC IDS

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the IDS and its monitor thread
        """
        logging.info("Stopping the OSSECIDS")
        monitor_running = self._is_monitor_running()
        result = subprocess.run(constants.OSSEC.STOP_OSSEC_IDS.split(" "), capture_output=True, text=True)
        logging.info(f"Stopped the OSSECIDS, stdout: {result.stdout}, stderr: {result.stderr}")
        return csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO(
            monitor_running=monitor_running, ossec_ids_running=False)

    def getOSSECIdsMonitorStatus(
            self, request: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.GetOSSECIdsMonitorStatusMsg,
            context: grpc.ServicerContext) -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO:
        """
        Gets the status of the OSSEC IDS Monitor thread

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the IDS monitor
        """
        monitor_running = self._is_monitor_running()
        ossec_ids_running = self._is_ossec_running()
        return csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO(
            monitor_running=monitor_running, ossec_ids_running=ossec_ids_running)


def serve(port: int = 50047, log_dir: str = "/", max_workers: int = 10,
          log_file_name: str = "ossec_ids_manager.log") -> None:
    """
    Starts the gRPC server for managing clients

    :param port: the port that the server will listen to
    :param log_dir: the directory to write the log file
    :param log_file_name: the file name of the log
    :param max_workers: the maximum number of GRPC workers
    :return: None
    """
    constants.LOG_FILES.OSSEC_IDS_MANAGER_LOG_DIR = log_dir
    constants.LOG_FILES.OSSEC_IDS_MANAGER_LOG_FILE = log_file_name
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers))
    csle_collector.ossec_ids_manager.ossec_ids_manager_pb2_grpc.add_OSSECIdsManagerServicer_to_server(
        OSSECIdsManagerServicer(), server)
    server.add_insecure_port(f'[::]:{port}')
    server.start()
    logging.info(f"OSSECIdsManager Server Started, Listening on port: {port}")
    server.wait_for_termination()


# Program entrypoint
if __name__ == '__main__':
    serve()



from typing import Dict, Any
import csle_collector.kafka_manager.kafka_manager_pb2


class KafkaManagerUtil:
    """
    Class with utility functions related to the Kafka Manager
    """

    @staticmethod
    def kafka_dto_to_dict(kafka_dto: csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO) -> Dict[str, Any]:
        """
        Converts a KafkaDTO to a dict

        :param kafka_dto: the dto to convert
        :return: a dict representation of the DTO
        """
        d: Dict[str, Any] = {}
        d["running"] = kafka_dto.running
        topics = []
        for top in kafka_dto.topics:
            topics.append(str(top))
        d["topics"] = topics
        return d

    @staticmethod
    def kafka_dto_from_dict(d: Dict[str, Any]) -> csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO:
        """
        Converts a dict representation of a KafkaDTO to a DTO

        :param d: the dict to convert
        :return: the converted DTO
        """
        kafka_dto = csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO(running=d["running"],
                                                                            topics=d["topics"])
        return kafka_dto

    @staticmethod
    def kafka_dto_empty() -> csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO:
        """
        :return: an empty KafkaDTO
        """
        kafka_dto = csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO()
        kafka_dto.running = False
        return kafka_dto

    @staticmethod
    def hours_to_ms(hours: float) -> float:
        """
        Convert hours to ms

        :param hours: the hours to convert
        :return: the ms
        """
        return int((((hours * 1000) * 60) * 60))



from typing import Any
import pytest
import pytest_mock
import csle_collector.docker_stats_manager.query_docker_stats_manager
from csle_collector.docker_stats_manager.docker_stats_manager_pb2 import DockerStatsMonitorDTO, ContainerIp
from csle_collector.docker_stats_manager.docker_stats_manager import DockerStatsManagerServicer


class TestDockerStatsManagerSuite:
    """
    Test suite for the Docker stats manager
    """

    @pytest.fixture(scope='module')
    def grpc_add_to_server(self) -> Any:
        """
        Necessary fixture for pytest-grpc

        :return: the add_servicer_to_server function
        """
        from csle_collector.docker_stats_manager.docker_stats_manager_pb2_grpc \
            import add_DockerStatsManagerServicer_to_server
        return add_DockerStatsManagerServicer_to_server

    @pytest.fixture(scope='module')
    def grpc_servicer(self) -> DockerStatsManagerServicer:
        """
        Necessary fixture for pytest-grpc

        :return: the docker stats manager servicer
        """
        return DockerStatsManagerServicer()

    @pytest.fixture(scope='module')
    def grpc_stub_cls(self, grpc_channel):
        """
        Necessary fixture for pytest-grpc

        :param grpc_channel: the grpc channel for testing
        :return: the stub to the service
        """
        from csle_collector.docker_stats_manager.docker_stats_manager_pb2_grpc import DockerStatsManagerStub
        return DockerStatsManagerStub

    def test_getDockerStatsMonitorStatus(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the getDockerStatsMonitorStatus grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('time.sleep', return_value=None)
        dsmt_1 = mocker.MagicMock()
        dsmt_1.stopped = True
        dsmt_1.emulation = "em1"
        dsmt_1.execution_first_ip_octet = 5
        dsmt_1.configure_mock(**{"is_alive.return_value": False})
        dsmt_2 = mocker.MagicMock()
        dsmt_2.stopped = False
        dsmt_2.emulation = "em2"
        dsmt_2.execution_first_ip_octet = 8
        dsmt_2.configure_mock(**{"is_alive.return_value": True})
        mocker.patch('csle_collector.docker_stats_manager.docker_stats_manager.DockerStatsManagerServicer.'
                     'get_docker_stats_monitor_threads', return_value=[dsmt_1, dsmt_2])
        response: DockerStatsMonitorDTO = (
            csle_collector.docker_stats_manager.query_docker_stats_manager.get_docker_stats_manager_status(
                stub=grpc_stub))
        assert response is not None
        assert response.num_monitors == 1
        assert len(response.emulations) == 1
        assert list(response.emulations) == ["em2"]
        assert len(list(response.emulation_executions)) == 1
        assert list(response.emulation_executions) == [8]

    def test_stopDockerStatsMonitor(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopDockerStatsMonitor grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('time.sleep', return_value=None)
        dsmt_1 = mocker.MagicMock()
        dsmt_1.stopped = False
        dsmt_1.emulation = "em1"
        dsmt_1.execution_first_ip_octet = 5
        dsmt_1.configure_mock(**{"is_alive.return_value": True})
        dsmt_2 = mocker.MagicMock()
        dsmt_2.stopped = False
        dsmt_2.emulation = "em2"
        dsmt_2.execution_first_ip_octet = 8
        dsmt_2.configure_mock(**{"is_alive.return_value": True})
        mocker.patch('csle_collector.docker_stats_manager.docker_stats_manager.DockerStatsManagerServicer.'
                     'get_docker_stats_monitor_threads', return_value=[dsmt_1, dsmt_2])
        response: DockerStatsMonitorDTO = (
            csle_collector.docker_stats_manager.query_docker_stats_manager.stop_docker_stats_monitor(
                stub=grpc_stub, emulation="em1", execution_first_ip_octet=5))
        assert response is not None
        assert response.num_monitors == 1
        assert len(response.emulations) == 1
        assert list(response.emulations) == ["em2"]
        assert len(list(response.emulation_executions)) == 1
        assert list(response.emulation_executions) == [8]

    def test_startDockerStatsMonitor(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startDockerStatsMonitor grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('time.sleep', return_value=None)
        dsmt_1 = mocker.MagicMock()
        dsmt_1.stopped = False
        dsmt_1.emulation = "em1"
        dsmt_1.execution_first_ip_octet = 5
        dsmt_1.configure_mock(**{"is_alive.return_value": True})
        dsmt_2 = mocker.MagicMock()
        dsmt_2.stopped = False
        dsmt_2.emulation = "em2"
        dsmt_2.execution_first_ip_octet = 8
        dsmt_2.configure_mock(**{"is_alive.return_value": True})
        mocker.patch('csle_collector.docker_stats_manager.docker_stats_manager.DockerStatsManagerServicer.'
                     'get_docker_stats_monitor_threads', return_value=[dsmt_1, dsmt_2])
        mocker.patch('csle_collector.docker_stats_manager.threads.docker_stats_thread.DockerStatsThread.__init__',
                     return_value=None)
        mocker.patch('csle_collector.docker_stats_manager.threads.docker_stats_thread.DockerStatsThread.run',
                     return_value=True)
        mocker.patch('csle_collector.docker_stats_manager.threads.docker_stats_thread.DockerStatsThread.start',
                     return_value=True)
        kafka_ip = "192.168.25.1"
        stats_queue_maxsize = 5
        time_step_len_seconds = 19
        kafka_port = 34
        containers = [ContainerIp(ip="7.7.7.7", container="container1")]
        response: DockerStatsMonitorDTO = (
            csle_collector.docker_stats_manager.query_docker_stats_manager.start_docker_stats_monitor(
                stub=grpc_stub, emulation="em3", execution_first_ip_octet=9, kafka_ip=kafka_ip,
                stats_queue_maxsize=stats_queue_maxsize, time_step_len_seconds=time_step_len_seconds,
                kafka_port=kafka_port, containers=containers))
        assert response is not None
        import logging
        logging.getLogger().info(response.emulations)
        assert response.num_monitors == 3
        assert len(response.emulations) == 3
        assert "em1" in list(response.emulations)
        assert "em2" in list(response.emulations)
        assert "em3" in list(response.emulations)
        assert len(list(response.emulation_executions)) == 3
        assert 5 in list(response.emulation_executions)
        assert 8 in list(response.emulation_executions)
        assert 9 in list(response.emulation_executions)



from typing import Any
import pytest
import pytest_mock
import csle_collector.elk_manager.query_elk_manager
from csle_collector.elk_manager.elk_manager_pb2 import ElkDTO
from csle_collector.elk_manager.elk_manager import ElkManagerServicer


class TestElkManagerSuite:
    """
    Test suite for the ELK manager
    """

    @pytest.fixture(scope='module')
    def grpc_add_to_server(self) -> Any:
        """
        Necessary fixture for pytest-grpc

        :return: the add_servicer_to_server function
        """
        from csle_collector.elk_manager.elk_manager_pb2_grpc import add_ElkManagerServicer_to_server
        return add_ElkManagerServicer_to_server

    @pytest.fixture(scope='module')
    def grpc_servicer(self) -> ElkManagerServicer:
        """
        Necessary fixture for pytest-grpc

        :return: the host manager servicer
        """
        return ElkManagerServicer()

    @pytest.fixture(scope='module')
    def grpc_stub_cls(self, grpc_channel):
        """
        Necessary fixture for pytest-grpc

        :param grpc_channel: the grpc channel for testing
        :return: the stub to the service
        """
        from csle_collector.elk_manager.elk_manager_pb2_grpc import ElkManagerStub
        return ElkManagerStub

    def test_getElkStatus(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopElk grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.elk_manager.elk_manager.ElkManagerServicer._get_elk_status',
                     return_value=(False, False, False))
        response: ElkDTO = csle_collector.elk_manager.query_elk_manager.get_elk_status(stub=grpc_stub)
        assert not response.elasticRunning
        assert not response.kibanaRunning
        assert not response.logstashRunning
        mocker.patch('csle_collector.elk_manager.elk_manager.ElkManagerServicer._get_elk_status',
                     return_value=(True, False, False))
        response = csle_collector.elk_manager.query_elk_manager.get_elk_status(stub=grpc_stub)
        assert response.elasticRunning
        assert not response.kibanaRunning
        assert not response.logstashRunning

        mocker.patch('csle_collector.elk_manager.elk_manager.ElkManagerServicer._get_elk_status',
                     return_value=(True, False, True))
        response = csle_collector.elk_manager.query_elk_manager.get_elk_status(stub=grpc_stub)
        assert response.elasticRunning
        assert not response.kibanaRunning
        assert response.logstashRunning

        mocker.patch('csle_collector.elk_manager.elk_manager.ElkManagerServicer._get_elk_status',
                     return_value=(True, True, True))
        response = csle_collector.elk_manager.query_elk_manager.get_elk_status(stub=grpc_stub)
        assert response.elasticRunning
        assert response.kibanaRunning
        assert response.logstashRunning

    def test_stopElk(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopElk grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('os.system', return_value=True)
        response: ElkDTO = csle_collector.elk_manager.query_elk_manager.stop_elk(stub=grpc_stub)
        assert not response.elasticRunning
        assert not response.kibanaRunning
        assert not response.logstashRunning

    def test_startElk(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopElk grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('os.system', return_value=True)
        response: ElkDTO = csle_collector.elk_manager.query_elk_manager.start_elk(stub=grpc_stub)
        assert response.elasticRunning
        assert response.kibanaRunning
        assert response.logstashRunning

    def test_startElastic(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startElastic grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('os.system', return_value=True)
        mocker.patch('csle_collector.elk_manager.elk_manager.ElkManagerServicer._get_elk_status',
                     return_value=(False, False, False))
        response: ElkDTO = csle_collector.elk_manager.query_elk_manager.start_elastic(stub=grpc_stub)
        assert response.elasticRunning
        assert not response.kibanaRunning
        assert not response.logstashRunning

    def test_stopElastic(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopElastic grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('os.system', return_value=True)
        mocker.patch('csle_collector.elk_manager.elk_manager.ElkManagerServicer._get_elk_status',
                     return_value=(True, True, True))
        response: ElkDTO = csle_collector.elk_manager.query_elk_manager.stop_elastic(stub=grpc_stub)
        assert not response.elasticRunning
        assert response.kibanaRunning
        assert response.logstashRunning

    def test_startKibana(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startKibana grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('os.system', return_value=True)
        mocker.patch('csle_collector.elk_manager.elk_manager.ElkManagerServicer._get_elk_status',
                     return_value=(False, False, False))
        response: ElkDTO = csle_collector.elk_manager.query_elk_manager.start_kibana(stub=grpc_stub)
        assert not response.elasticRunning
        assert response.kibanaRunning
        assert not response.logstashRunning

    def test_stopKibana(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopKibana grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('os.system', return_value=True)
        mocker.patch('csle_collector.elk_manager.elk_manager.ElkManagerServicer._get_elk_status',
                     return_value=(True, True, True))
        response: ElkDTO = csle_collector.elk_manager.query_elk_manager.stop_kibana(stub=grpc_stub)
        assert response.elasticRunning
        assert not response.kibanaRunning
        assert response.logstashRunning

    def test_startLogstash(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startLogstash grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('os.system', return_value=True)
        mocker.patch('csle_collector.elk_manager.elk_manager.ElkManagerServicer._get_elk_status',
                     return_value=(False, False, False))
        response: ElkDTO = csle_collector.elk_manager.query_elk_manager.start_logstash(stub=grpc_stub)
        assert not response.elasticRunning
        assert not response.kibanaRunning
        assert response.logstashRunning

    def test_stopLogstash(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopLogstash grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('os.system', return_value=True)
        mocker.patch('csle_collector.elk_manager.elk_manager.ElkManagerServicer._get_elk_status',
                     return_value=(True, True, True))
        response: ElkDTO = csle_collector.elk_manager.query_elk_manager.stop_logstash(stub=grpc_stub)
        assert response.elasticRunning
        assert response.kibanaRunning
        assert not response.logstashRunning



from typing import List
import threading
import time
import logging
import math
from scipy.stats import poisson
from csle_collector.client_manager.threads.client_thread import ClientThread
from csle_collector.client_manager.dao.client import Client
from csle_collector.client_manager.dao.workflows_config import WorkflowsConfig
from csle_collector.client_manager.dao.client_arrival_type import ClientArrivalType
from csle_collector.client_manager.dao.constant_arrival_config import ConstantArrivalConfig
from csle_collector.client_manager.dao.sine_arrival_config import SineArrivalConfig
from csle_collector.client_manager.dao.piece_wise_constant_arrival_config import PieceWiseConstantArrivalConfig
from csle_collector.client_manager.dao.spiking_arrival_config import SpikingArrivalConfig
from csle_collector.client_manager.dao.eptmp_arrival_config import EPTMPArrivalConfig


class ArrivalThread(threading.Thread):
    """
    Thread that generates client arrivals (starts client threads according to a Poisson process)
    """

    def __init__(self, time_step_len_seconds: float, clients: List[Client], workflows_config: WorkflowsConfig):
        """
        Initializes the arrival thread

        :param time_step_len_seconds: the number of seconds that one time-unit of the Poisson process corresponds to
        :param clients: the list of client profiles
        :param workflows_config: the workflow configurations
        """
        threading.Thread.__init__(self)
        self.time_step_len_seconds = time_step_len_seconds
        self.client_threads: List[ClientThread] = []
        self.t = 0
        self.clients = clients
        self.workflows_config = workflows_config
        self.stopped = False
        self.rate = 0.0
        logging.info(f"Starting arrival thread, num client types:{len(self.clients)}, "
                     f"num workflows: {len(self.workflows_config.workflow_markov_chains)}")

    @staticmethod
    def piece_wise_constant_rate(t, arrival_config: PieceWiseConstantArrivalConfig) -> float:
        """
        Function that returns the rate of a piece-wise constant Poisson process

        :param t: the time-step
        :param arrival_config: the arrival process configuration
        :return: the rate
        """
        rate = 0.0
        assert len(arrival_config.breakvalues) == len(arrival_config.breakpoints)
        for i in range(len(arrival_config.breakvalues)):
            if t >= arrival_config.breakpoints[i]:
                rate = arrival_config.breakvalues[i]
        return rate

    @staticmethod
    def spiking_poisson_arrival_rate(t, arrival_config: SpikingArrivalConfig) -> float:
        """
        Function that returns the rate of a spiking Poisson process

        :param t: the time-step
        :param arrival_config: the arrival process configuration
        :return: the rate
        """
        assert len(arrival_config.exponents) == len(arrival_config.factors)
        rate = 0.0
        for i in range(len(arrival_config.exponents)):
            rate = arrival_config.factors[i] * math.exp(math.pow(-(t - arrival_config.exponents[i]), 2))
        return rate

    @staticmethod
    def sine_modulated_poisson_rate(t, arrival_config: SineArrivalConfig) -> float:
        """
        Function that returns the rate of a sine-modulated Poisson process

        :param t: the time-step
        :param arrival_config: the arrival process configuration
        :return: the rate
        """
        return arrival_config.lamb + arrival_config.period_scaling_factor * math.sin(
            arrival_config.time_scaling_factor * math.pi * t)

    @staticmethod
    def constant_poisson_rate(arrival_config: ConstantArrivalConfig) -> float:
        """
        Function that returns the rate of a stationary Poisson process

        :param arrival_config: the arrival process configuration
        :return: the rate
        """
        return arrival_config.lamb

    @staticmethod
    def eptmp_rate(t, arrival_config: EPTMPArrivalConfig) -> float:
        """
        Function that returns the rate of a EPTMP Poisson process.

        EPTMP or Exponential-Polynomial-Trigonometric rate function having Multiple Periodicities.
        This class is used for creating a rate function that can exhibit both global trends as well as
        periodic components with individual frequencies and amplitudes.
        (Kuhl and Wilson, 1995)

        :param t: the time-step
        :param arrival_config: the arrival process configuration
        :return: the rate
        """
        theta_sum = 0.0
        for i, theta in enumerate(arrival_config.thetas):
            theta_sum += theta * pow(t, i)
        second_sum = 0.0
        for i, (gamma, phi, omega) in enumerate(zip(arrival_config.gammas, arrival_config.phis, arrival_config.omegas)):
            second_sum += gamma * math.sin(omega * t + phi)
        return math.exp(theta_sum + second_sum)

    def run(self) -> None:
        """
        Runs the arrival generator, generates new clients dynamically according to a Poisson process

        :return: None
        """
        while not self.stopped:
            new_client_threads = []
            for ct in self.client_threads:
                if ct.is_alive():
                    new_client_threads.append(ct)
            self.client_threads = new_client_threads
            self.t += 1
            num_new_clients = 0
            for c in self.clients:
                try:
                    if c.arrival_config.client_arrival_type.value == ClientArrivalType.SINE_MODULATED.value:
                        self.rate = self.sine_modulated_poisson_rate(t=self.t, arrival_config=c.arrival_config)
                        num_new_clients = poisson.rvs(self.rate, size=1)[0]
                    elif c.arrival_config.client_arrival_type.value == ClientArrivalType.CONSTANT.value:
                        self.rate = self.constant_poisson_rate(arrival_config=c.arrival_config)
                        num_new_clients = poisson.rvs(self.rate, size=1)[0]
                    elif c.arrival_config.client_arrival_type.value == ClientArrivalType.PIECE_WISE_CONSTANT.value:
                        self.rate = self.piece_wise_constant_rate(t=self.t, arrival_config=c.arrival_config)
                        num_new_clients = poisson.rvs(self.rate, size=1)[0]
                    elif c.arrival_config.client_arrival_type.value == ClientArrivalType.EPTMP.value:
                        self.rate = self.eptmp_rate(t=self.t, arrival_config=c.arrival_config)
                        num_new_clients = poisson.rvs(self.rate, size=1)[0]
                    else:
                        raise ValueError(f"Client arrival type: {c.arrival_config.client_arrival_type} not recognized")
                except Exception as e:
                    logging.info(f"There was an error computing the arrival rate: {str(e)}, {repr(e)}")
                try:
                    for nc in range(num_new_clients):
                        commands = c.generate_commands(workflows_config=self.workflows_config)
                        thread = ClientThread(commands=commands, time_step_len_seconds=self.time_step_len_seconds)
                        thread.start()
                        self.client_threads.append(thread)
                except Exception as e:
                    logging.info(f"There was an error starting the client threads: {str(e), repr(e)}")
            time.sleep(self.time_step_len_seconds)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: traffic_manager.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15traffic_manager.proto\"\x10\n\x0eStopTrafficMsg\"6\n\x0fStartTrafficMsg\x12\x11\n\tsleepTime\x18\x01 \x01(\x05\x12\x10\n\x08\x63ommands\x18\x02 \x03(\t\"\x15\n\x13GetTrafficStatusMsg\"-\n\nTrafficDTO\x12\x0f\n\x07running\x18\x01 \x01(\x08\x12\x0e\n\x06script\x18\x02 \x01(\t2\xa9\x01\n\x0eTrafficManager\x12\x37\n\x10getTrafficStatus\x12\x14.GetTrafficStatusMsg\x1a\x0b.TrafficDTO\"\x00\x12-\n\x0bstopTraffic\x12\x0f.StopTrafficMsg\x1a\x0b.TrafficDTO\"\x00\x12/\n\x0cstartTraffic\x12\x10.StartTrafficMsg\x1a\x0b.TrafficDTO\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'traffic_manager_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STOPTRAFFICMSG._serialized_start=25
  _STOPTRAFFICMSG._serialized_end=41
  _STARTTRAFFICMSG._serialized_start=43
  _STARTTRAFFICMSG._serialized_end=97
  _GETTRAFFICSTATUSMSG._serialized_start=99
  _GETTRAFFICSTATUSMSG._serialized_end=120
  _TRAFFICDTO._serialized_start=122
  _TRAFFICDTO._serialized_end=167
  _TRAFFICMANAGER._serialized_start=170
  _TRAFFICMANAGER._serialized_end=339
# @@protoc_insertion_point(module_scope)



from typing import Tuple
import logging
import os
import subprocess
from concurrent import futures
import grpc
import socket
import netifaces
import csle_collector.elk_manager.elk_manager_pb2_grpc
import csle_collector.elk_manager.elk_manager_pb2
import csle_collector.constants.constants as constants


class ElkManagerServicer(csle_collector.elk_manager.elk_manager_pb2_grpc.ElkManagerServicer):
    """
    gRPC server for managing an ELK stack. Allows to start/stop the ELK stack remotely and also to query the
    state of the ELK stack.
    """

    def __init__(self) -> None:
        """
        Initializes the server
        """
        logging.basicConfig(filename=f"{constants.LOG_FILES.ELK_MANAGER_LOG_DIR}"
                                     f"{constants.LOG_FILES.ELK_MANAGER_LOG_FILE}", level=logging.INFO)
        self.hostname = socket.gethostname()
        try:
            self.ip = netifaces.ifaddresses(constants.INTERFACES.ETH0)[netifaces.AF_INET][0][constants.INTERFACES.ADDR]
        except Exception:
            self.ip = socket.gethostbyname(self.hostname)
        logging.info(f"Setting up ElkManager hostname: {self.hostname} ip: {self.ip}")

    def _get_elk_status(self) -> Tuple[bool, bool, bool]:
        """
        Utility method to get the status of the ELK stack

        :return: status of elastic, status of kibana, status of logstash
        """
        status_output = subprocess.run(constants.ELK.ELASTICSEARCH_STATUS.split(" "),
                                       capture_output=True, text=True).stdout
        elasticsearch_running = not ("not" in status_output)
        status_output = subprocess.run(constants.ELK.KIBANA_STATUS.split(" "),
                                       capture_output=True, text=True).stdout
        kibana_running = not ("not" in status_output)
        status_output = subprocess.run(constants.ELK.LOGSTASH_STATUS.split(" "),
                                       capture_output=True, text=True).stdout
        logstash_running = not ("not" in status_output)
        return elasticsearch_running, kibana_running, logstash_running

    def getElkStatus(self, request: csle_collector.elk_manager.elk_manager_pb2.GetElkStatusMsg,
                     context: grpc.ServicerContext) \
            -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
        """
        Gets the state of the ELK server

        :param request: the gRPC request
        :param context: the gRPC context
        :return: an ElkDTO with the state of the ELK stack server
        """
        elasticsearch_running, kibana_running, logstash_running = self._get_elk_status()
        elk_dto = csle_collector.elk_manager.elk_manager_pb2.ElkDTO(elasticRunning=elasticsearch_running,
                                                                    kibanaRunning=kibana_running,
                                                                    logstashRunning=logstash_running)
        return elk_dto

    def stopElk(self, request: csle_collector.elk_manager.elk_manager_pb2.StopElkMsg, context: grpc.ServicerContext) \
            -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
        """
        Stops the ELK stack

        :param request: the gRPC request
        :param context: the gRPC context
        :return: an ElkDTO with the state of the ELK stack
        """
        logging.info("Stopping the ELK stack")
        logging.info("Stopping kibana")
        os.system(constants.ELK.KIBANA_STOP)
        logging.info("Stopping logstash")
        os.system(constants.ELK.LOGSTASH_STOP)
        logging.info("Stopping elasticsearch")
        os.system(constants.ELK.ELASTICSEARCH_STOP)
        return csle_collector.elk_manager.elk_manager_pb2.ElkDTO(elasticRunning=False, kibanaRunning=False,
                                                                 logstashRunning=False)

    def startElk(self, request: csle_collector.elk_manager.elk_manager_pb2.StartElkMsg,
                 context: grpc.ServicerContext) -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
        """
        Starts the ELK stack

        :param request: the gRPC request
        :param context: the gRPC context
        :return: an ElkDTO with the state of the ELK server
        """
        logging.info("Starting ELK")
        os.system(constants.ELK.ELK_START)
        elk_dto = csle_collector.elk_manager.elk_manager_pb2.ElkDTO(elasticRunning=True, kibanaRunning=True,
                                                                    logstashRunning=True)
        return elk_dto

    def startElastic(self, request: csle_collector.elk_manager.elk_manager_pb2.StartElasticMsg,
                     context: grpc.ServicerContext) -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
        """
        Starts elasticsearch

        :param request: the gRPC request
        :param context: the gRPC context
        :return: an ElkDTO with the state of the ELK server
        """
        logging.info("Starting Elasticsearch")
        os.system(constants.ELK.ELASTICSEARCH_START)
        elasticsearch_running, kibana_running, logstash_running = self._get_elk_status()
        elk_dto = csle_collector.elk_manager.elk_manager_pb2.ElkDTO(elasticRunning=True, kibanaRunning=kibana_running,
                                                                    logstashRunning=logstash_running)
        return elk_dto

    def startKibana(self, request: csle_collector.elk_manager.elk_manager_pb2.StartKibanaMsg,
                    context: grpc.ServicerContext) -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
        """
        Starts Kibana

        :param request: the gRPC request
        :param context: the gRPC context
        :return: an ElkDTO with the state of the ELK server
        """
        logging.info("Starting Kibana")
        os.system(constants.ELK.KIBANA_START)
        elasticsearch_running, kibana_running, logstash_running = self._get_elk_status()
        elk_dto = csle_collector.elk_manager.elk_manager_pb2.ElkDTO(elasticRunning=elasticsearch_running,
                                                                    kibanaRunning=True,
                                                                    logstashRunning=logstash_running)
        return elk_dto

    def startLogstash(self, request: csle_collector.elk_manager.elk_manager_pb2.StartLogstashMsg,
                      context: grpc.ServicerContext) -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
        """
        Starts Logstash

        :param request: the gRPC request
        :param context: the gRPC context
        :return: an ElkDTO with the state of the ELK server
        """
        logging.info("Starting Logstash")
        os.system(constants.ELK.LOGSTASH_START)
        elasticsearch_running, kibana_running, logstash_running = self._get_elk_status()
        elk_dto = csle_collector.elk_manager.elk_manager_pb2.ElkDTO(elasticRunning=elasticsearch_running,
                                                                    kibanaRunning=kibana_running,
                                                                    logstashRunning=True)
        return elk_dto

    def stopElastic(self, request: csle_collector.elk_manager.elk_manager_pb2.StartElasticMsg,
                    context: grpc.ServicerContext) -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
        """
        Stops elasticsearch

        :param request: the gRPC request
        :param context: the gRPC context
        :return: an ElkDTO with the state of the ELK server
        """
        logging.info("Stops Elasticsearch")
        os.system(constants.ELK.ELASTICSEARCH_STOP)
        elasticsearch_running, kibana_running, logstash_running = self._get_elk_status()
        elk_dto = csle_collector.elk_manager.elk_manager_pb2.ElkDTO(elasticRunning=False, kibanaRunning=kibana_running,
                                                                    logstashRunning=logstash_running)
        return elk_dto

    def stopKibana(self, request: csle_collector.elk_manager.elk_manager_pb2.StopKibanaMsg,
                   context: grpc.ServicerContext) -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
        """
        Stops Kibana

        :param request: the gRPC request
        :param context: the gRPC context
        :return: an ElkDTO with the state of the ELK server
        """
        logging.info("Stops Kibana")
        os.system(constants.ELK.KIBANA_STOP)
        elasticsearch_running, kibana_running, logstash_running = self._get_elk_status()
        elk_dto = csle_collector.elk_manager.elk_manager_pb2.ElkDTO(elasticRunning=elasticsearch_running,
                                                                    kibanaRunning=False,
                                                                    logstashRunning=logstash_running)
        return elk_dto

    def stopLogstash(self, request: csle_collector.elk_manager.elk_manager_pb2.StopLogstashMsg,
                     context: grpc.ServicerContext) -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
        """
        Stops Logstash

        :param request: the gRPC request
        :param context: the gRPC context
        :return: an ElkDTO with the state of the ELK server
        """
        logging.info("Stopping Logstash")
        os.system(constants.ELK.LOGSTASH_STOP)
        elasticsearch_running, kibana_running, logstash_running = self._get_elk_status()
        elk_dto = csle_collector.elk_manager.elk_manager_pb2.ElkDTO(elasticRunning=elasticsearch_running,
                                                                    kibanaRunning=kibana_running,
                                                                    logstashRunning=False)
        return elk_dto


def serve(port: int = 50045, log_dir: str = "/", max_workers: int = 10,
          log_file_name: str = "elk_manager.log") -> None:
    """
    Starts the gRPC server for managing the ELK stack

    :param port: the port that the server will listen to
    :param log_dir: the directory to write the log file
    :param log_file_name: the file name of the log
    :param max_workers: the maximum number of GRPC workers
    :return: None
    """
    constants.LOG_FILES.ELK_MANAGER_LOG_DIR = log_dir
    constants.LOG_FILES.ELK_MANAGER_LOG_FILE = log_file_name
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers))
    csle_collector.elk_manager.elk_manager_pb2_grpc.add_ElkManagerServicer_to_server(
        ElkManagerServicer(), server)
    server.add_insecure_port(f'[::]:{port}')
    server.start()
    logging.info(f"ElkManager Server Started, Listening on port: {port}")
    server.wait_for_termination()


# Program entrypoint
if __name__ == '__main__':
    serve()



from typing import Dict, Any
from csle_collector.client_manager.dao.client_arrival_type import ClientArrivalType
from csle_base.json_serializable import JSONSerializable


class ArrivalConfig(JSONSerializable):
    """
    Abstract arrival configuration class
    """

    def __init__(self, client_arrival_type: ClientArrivalType) -> None:
        """
        Initializes the object

        :param client_arrival_type: the type of the arrival configuration
        """
        self.client_arrival_type = client_arrival_type

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation

        :return: a dict representation of the object
        """
        d = {}
        d["client_arrival_type"] = self.client_arrival_type
        return d

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "ArrivalConfig":
        """
        Converts a dict representation to an instance

        :param d: the dict to convert
        :return: the created instance
        """
        obj = ArrivalConfig(client_arrival_type=d["client_arrival_type"])
        return obj

    @staticmethod
    def from_json_file(json_file_path: str) -> "ArrivalConfig":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return ArrivalConfig.from_dict(json.loads(json_str))



from typing import List
import os
import logging
from concurrent import futures
import grpc
import socket
import csle_collector.docker_stats_manager.docker_stats_manager_pb2_grpc
import csle_collector.docker_stats_manager.docker_stats_manager_pb2
import csle_collector.constants.constants as constants
from csle_collector.docker_stats_manager.threads.docker_stats_thread import DockerStatsThread


class DockerStatsManagerServicer(csle_collector.docker_stats_manager.docker_stats_manager_pb2_grpc.
                                 DockerStatsManagerServicer):
    """
    gRPC server for managing a docker stats monitor server.
    Allows to start/stop the docker stats monitor remotely and also to query the
    state of the server.
    """

    def __init__(self) -> None:
        """
        Initializes the server
        """
        file_name = constants.LOG_FILES.DOCKER_STATS_MANAGER_LOG_FILE
        dir = constants.LOG_FILES.DOCKER_STATS_MANAGER_LOG_DIR
        logfile = os.path.join(dir, file_name)
        logging.basicConfig(filename=logfile, level=logging.INFO)
        self.docker_stats_monitor_threads: List[DockerStatsThread] = []
        self.hostname = socket.gethostname()
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(("8.8.8.8", 80))
        self.ip = s.getsockname()[0]
        logging.info(f"Setting up DockerStatsManager, hostname: {self.hostname}, ip: {self.ip}")

    def get_docker_stats_monitor_threads(self) -> List[DockerStatsThread]:
        """
        Gets the list of Docker stats monitor threads

        :return: the list of Docker stats monitor threads
        """
        return self.docker_stats_monitor_threads

    def getDockerStatsMonitorStatus(
            self, request: csle_collector.docker_stats_manager.docker_stats_manager_pb2.GetDockerStatsMonitorStatusMsg,
            context: grpc.ServicerContext) \
            -> csle_collector.docker_stats_manager.docker_stats_manager_pb2.DockerStatsMonitorDTO:
        """
        Gets the state of the docker stats monitors

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the docker stats manager
        """
        new_docker_stats_monitor_threads = []
        emulations = []
        emulation_executions = []
        docker_stats_monitor_threads = self.get_docker_stats_monitor_threads()
        for dsmt in docker_stats_monitor_threads:
            if dsmt.is_alive() and not dsmt.stopped:
                new_docker_stats_monitor_threads.append(dsmt)
                emulations.append(dsmt.emulation)
                emulation_executions.append(dsmt.execution_first_ip_octet)
            else:
                dsmt.stopped = True
        emulations = list(set(emulations))
        emulation_executions = list(set(emulation_executions))
        self.docker_stats_monitor_threads = new_docker_stats_monitor_threads
        docker_stats_monitor_dto = csle_collector.docker_stats_manager.docker_stats_manager_pb2.DockerStatsMonitorDTO(
            num_monitors=len(new_docker_stats_monitor_threads), emulations=emulations,
            emulation_executions=emulation_executions)
        return docker_stats_monitor_dto

    def stopDockerStatsMonitor(
            self, request: csle_collector.docker_stats_manager.docker_stats_manager_pb2.StopDockerStatsMonitorMsg,
            context: grpc.ServicerContext):
        """
        Stops the docker stats monitor server

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the docker stats monitor server
        """
        logging.info(f"Stopping the docker stats monitor for emulation:{request.emulation}")

        new_docker_stats_monitor_threads = []
        emulations = []
        emulation_executions = []
        docker_stats_monitor_threads = self.get_docker_stats_monitor_threads()
        for dsmt in docker_stats_monitor_threads:
            if dsmt.emulation == request.emulation \
                    and dsmt.execution_first_ip_octet == request.execution_first_ip_octet:
                dsmt.stopped = True
            else:
                if dsmt.is_alive() and not dsmt.stopped:
                    new_docker_stats_monitor_threads.append(dsmt)
                    emulations.append(dsmt.emulation)
                    emulation_executions.append(dsmt.execution_first_ip_octet)
        self.docker_stats_monitor_threads = new_docker_stats_monitor_threads
        emulations = list(set(emulations))
        emulation_executions = list(set(emulation_executions))
        return csle_collector.docker_stats_manager.docker_stats_manager_pb2.DockerStatsMonitorDTO(
            num_monitors=len(new_docker_stats_monitor_threads), emulations=emulations,
            emulation_executions=emulation_executions)

    def startDockerStatsMonitor(
            self, request: csle_collector.docker_stats_manager.docker_stats_manager_pb2.StartDockerStatsMonitorMsg,
            context: grpc.ServicerContext) \
            -> csle_collector.docker_stats_manager.docker_stats_manager_pb2.DockerStatsMonitorDTO:
        """
        Starts a new docker stats monitor

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the docker stats monitor
        """
        logging.info(f"Starting the docker stats monitor for emulation:{request.emulation}")

        # Stop any existing thread with the same name
        new_docker_stats_monitor_threads = []
        emulations = []
        emulation_executions = []
        docker_stats_monitor_threads = self.get_docker_stats_monitor_threads()
        for dsmt in docker_stats_monitor_threads:
            if dsmt.is_alive() and not dsmt.stopped:
                new_docker_stats_monitor_threads.append(dsmt)
                emulations.append(dsmt.emulation)
                emulation_executions.append(dsmt.execution_first_ip_octet)
        docker_stats_monitor_thread = DockerStatsThread(list(request.containers), request.emulation,
                                                        request.execution_first_ip_octet, request.kafka_ip,
                                                        request.stats_queue_maxsize, request.time_step_len_seconds,
                                                        request.kafka_port)
        docker_stats_monitor_thread.start()
        new_docker_stats_monitor_threads.append(docker_stats_monitor_thread)
        emulations.append(request.emulation)
        emulation_executions.append(request.execution_first_ip_octet)
        emulations = list(set(emulations))
        emulation_executions = list(set(emulation_executions))
        self.docker_stats_monitor_threads = new_docker_stats_monitor_threads
        return csle_collector.docker_stats_manager.docker_stats_manager_pb2.DockerStatsMonitorDTO(
            num_monitors=len(new_docker_stats_monitor_threads), emulations=emulations,
            emulation_executions=emulation_executions)


def serve(port: int = 50046, log_dir: str = "/var/log/csle/", max_workers: int = 10,
          log_file_name: str = "docker_stats_manager.log") -> None:
    """
    Starts the gRPC server for managing docker stats collection

    :param port: the port that the server will listen to
    :param log_dir: the directory to write the log file
    :param log_file_name: the file name of the log
    :param max_workers: the maximum number of parallel gRPC workers
    :return: None
    """
    constants.LOG_FILES.DOCKER_STATS_MANAGER_LOG_DIR = log_dir
    constants.LOG_FILES.DOCKER_STATS_MANAGER_LOG_FILE = log_file_name
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers))
    csle_collector.docker_stats_manager.docker_stats_manager_pb2_grpc.add_DockerStatsManagerServicer_to_server(
        DockerStatsManagerServicer(), server)
    server.add_insecure_port(f'[::]:{port}')
    server.start()
    logging.info(f"DockerStatsManager Server Started, Listening on port: {port}")
    server.wait_for_termination()


# Program entrypoint
if __name__ == '__main__':
    serve()



from typing import Any
import pytest
import pytest_mock
import csle_collector.client_manager.query_clients
from csle_collector.client_manager.client_manager_pb2 import ClientsDTO
from csle_collector.client_manager.client_manager import ClientManagerServicer
from csle_collector.client_manager.dao.workflows_config import WorkflowsConfig
from csle_collector.client_manager.dao.workflow_markov_chain import WorkflowMarkovChain
from csle_collector.client_manager.dao.workflow_service import WorkflowService
from csle_collector.client_manager.dao.client import Client
from csle_collector.client_manager.dao.sine_arrival_config import SineArrivalConfig
from csle_collector.client_manager.dao.constant_arrival_config import ConstantArrivalConfig
from csle_collector.client_manager.dao.eptmp_arrival_config import EPTMPArrivalConfig
from csle_collector.client_manager.dao.spiking_arrival_config import SpikingArrivalConfig
from csle_collector.client_manager.dao.piece_wise_constant_arrival_config import PieceWiseConstantArrivalConfig


class TestClientManagerSuite:
    """
    Test suite for the Client manager
    """

    @pytest.fixture(scope='module')
    def grpc_add_to_server(self) -> Any:
        """
        Necessary fixture for pytest-grpc

        :return: the add_servicer_to_server function
        """
        from csle_collector.client_manager.client_manager_pb2_grpc import add_ClientManagerServicer_to_server
        return add_ClientManagerServicer_to_server

    @pytest.fixture(scope='module')
    def grpc_servicer(self) -> ClientManagerServicer:
        """
        Necessary fixture for pytest-grpc

        :return: the client manager servicer
        """
        return ClientManagerServicer()

    @pytest.fixture(scope='module')
    def grpc_stub_cls(self, grpc_channel):
        """
        Necessary fixture for pytest-grpc

        :param grpc_channel: the grpc channel for testing
        :return: the stub to the service
        """
        from csle_collector.client_manager.client_manager_pb2_grpc import ClientManagerStub
        return ClientManagerStub

    def test_getClients(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the getClients grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('time.sleep', return_value=None)
        arrival_thread = mocker.MagicMock()
        arrival_thread.client_threads = ["client1", "client2"]
        arrival_thread.time_step_len_seconds = 30
        producer_thread = mocker.MagicMock()
        producer_thread.time_step_len_seconds = 30
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_producer_thread',
                     return_value=producer_thread)
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_arrival_thread',
                     return_value=arrival_thread)
        response: ClientsDTO = csle_collector.client_manager.query_clients.get_clients(stub=grpc_stub)
        assert response is not None
        assert response.num_clients == len(arrival_thread.client_threads)
        assert response.client_process_active
        assert response.producer_active
        assert response.clients_time_step_len_seconds == 30
        assert response.producer_time_step_len_seconds == 30
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_producer_thread',
                     return_value=None)
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_arrival_thread',
                     return_value=None)
        response = csle_collector.client_manager.query_clients.get_clients(stub=grpc_stub)
        assert response is not None
        assert response.num_clients == 0
        assert not response.client_process_active
        assert not response.producer_active
        assert response.clients_time_step_len_seconds == 0.0
        assert response.producer_time_step_len_seconds == 0.0

    def test_stopClients(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopClients grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        arrival_thread = mocker.MagicMock()
        arrival_thread.client_threads = ["client1", "client2"]
        arrival_thread.time_step_len_seconds = 30
        producer_thread = mocker.MagicMock()
        producer_thread.time_step_len_seconds = 30
        mocker.patch('time.sleep', return_value=None)
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_producer_thread',
                     return_value=producer_thread)
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_arrival_thread',
                     return_value=arrival_thread)
        response: ClientsDTO = csle_collector.client_manager.query_clients.stop_clients(stub=grpc_stub)
        assert response is not None
        assert response.num_clients == 0
        assert not response.client_process_active
        assert response.producer_active
        assert response.clients_time_step_len_seconds == 30
        assert response.producer_time_step_len_seconds == 30

        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_producer_thread',
                     return_value=None)
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_arrival_thread',
                     return_value=None)
        response = csle_collector.client_manager.query_clients.stop_clients(stub=grpc_stub)
        assert response is not None
        assert response.num_clients == 0
        assert not response.client_process_active
        assert not response.producer_active
        assert response.clients_time_step_len_seconds == 0.0
        assert response.producer_time_step_len_seconds == 0.0

    def test_startClients(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startClients grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        arrival_thread = mocker.MagicMock()
        arrival_thread.client_threads = ["client1", "client2"]
        arrival_thread.time_step_len_seconds = 30
        producer_thread = mocker.MagicMock()
        producer_thread.time_step_len_seconds = 30
        mocker.patch('time.sleep', return_value=None)
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_producer_thread',
                     return_value=producer_thread)
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_arrival_thread',
                     return_value=arrival_thread)
        mocker.patch('csle_collector.client_manager.threads.arrival_thread.ArrivalThread.__init__',
                     return_value=None)
        mocker.patch('csle_collector.client_manager.threads.arrival_thread.ArrivalThread.run',
                     return_value=True)
        mocker.patch('csle_collector.client_manager.threads.arrival_thread.ArrivalThread.start',
                     return_value=True)
        time_step_len_seconds = 30
        workflows_config = WorkflowsConfig(
            workflow_markov_chains=[
                WorkflowMarkovChain(transition_matrix=[[0.2, 0.8], [0.8, 0.2]], initial_state=0, id=0)],
            workflow_services=[WorkflowService(ips_and_commands=[("192.168.1.1", ["ping localhost"])], id=0)])
        response: ClientsDTO = csle_collector.client_manager.query_clients.start_clients(
            stub=grpc_stub, time_step_len_seconds=time_step_len_seconds, workflows_config=workflows_config,
            clients=[Client(id=0, workflow_distribution=[0.9, 0.1],
                            arrival_config=SineArrivalConfig(
                                lamb=9.0, time_scaling_factor=2.0, period_scaling_factor=5.0),
                            mu=4, exponential_service_time=False)])
        assert response is not None
        assert response.num_clients == 0
        assert response.client_process_active
        assert response.producer_active
        assert response.clients_time_step_len_seconds == time_step_len_seconds
        assert response.producer_time_step_len_seconds == 30

        workflows_config = WorkflowsConfig(
            workflow_markov_chains=[
                WorkflowMarkovChain(transition_matrix=[[0.2, 0.8], [0.8, 0.2]], initial_state=0, id=0)],
            workflow_services=[WorkflowService(ips_and_commands=[("192.168.1.1", ["ping localhost"])], id=0)])
        response = csle_collector.client_manager.query_clients.start_clients(
            stub=grpc_stub, time_step_len_seconds=time_step_len_seconds, workflows_config=workflows_config,
            clients=[Client(id=0, workflow_distribution=[0.9, 0.1],
                            arrival_config=SpikingArrivalConfig(exponents=[0.2, 0.5], factors=[0.9, 0.1]),
                            mu=4, exponential_service_time=False)])
        assert response is not None
        assert response.num_clients == 0
        assert response.client_process_active
        assert response.producer_active
        assert response.clients_time_step_len_seconds == time_step_len_seconds
        assert response.producer_time_step_len_seconds == 30

        workflows_config = WorkflowsConfig(
            workflow_markov_chains=[
                WorkflowMarkovChain(transition_matrix=[[0.2, 0.8], [0.8, 0.2]], initial_state=0, id=0)],
            workflow_services=[WorkflowService(ips_and_commands=[("192.168.1.1", ["ping localhost"])], id=0)])
        response = csle_collector.client_manager.query_clients.start_clients(
            stub=grpc_stub, time_step_len_seconds=time_step_len_seconds, workflows_config=workflows_config,
            clients=[Client(id=0, workflow_distribution=[0.9, 0.1],
                            arrival_config=ConstantArrivalConfig(lamb=9.0),
                            mu=4, exponential_service_time=False)])
        assert response is not None
        assert response.num_clients == 0
        assert response.client_process_active
        assert response.producer_active
        assert response.clients_time_step_len_seconds == time_step_len_seconds
        assert response.producer_time_step_len_seconds == 30

        workflows_config = WorkflowsConfig(
            workflow_markov_chains=[
                WorkflowMarkovChain(transition_matrix=[[0.2, 0.8], [0.8, 0.2]], initial_state=0, id=0)],
            workflow_services=[WorkflowService(ips_and_commands=[("192.168.1.1", ["ping localhost"])], id=0)])
        response = csle_collector.client_manager.query_clients.start_clients(
            stub=grpc_stub, time_step_len_seconds=time_step_len_seconds, workflows_config=workflows_config,
            clients=[Client(id=0, workflow_distribution=[0.9, 0.1],
                            arrival_config=EPTMPArrivalConfig(
                                thetas=[0.15, 0.6, 0.215], gammas=[0.1, 0.6, 0.8], phis=[0.16, 0.52, 0.1],
                                omegas=[0.1, 0.6, 0.9]),
                            mu=4, exponential_service_time=False)])
        assert response is not None
        assert response.num_clients == 0
        assert response.client_process_active
        assert response.producer_active
        assert response.clients_time_step_len_seconds == time_step_len_seconds
        assert response.producer_time_step_len_seconds == 30

        workflows_config = WorkflowsConfig(
            workflow_markov_chains=[
                WorkflowMarkovChain(transition_matrix=[[0.2, 0.8], [0.8, 0.2]], initial_state=0, id=0)],
            workflow_services=[WorkflowService(ips_and_commands=[("192.168.1.1", ["ping localhost"])], id=0)])
        response = csle_collector.client_manager.query_clients.start_clients(
            stub=grpc_stub, time_step_len_seconds=time_step_len_seconds, workflows_config=workflows_config,
            clients=[Client(id=0, workflow_distribution=[0.9, 0.1],
                            arrival_config=PieceWiseConstantArrivalConfig(
                                breakvalues=[0.65, 0.12, 0.9], breakpoints=[1, 5, 10]),
                            mu=4, exponential_service_time=False)])
        assert response is not None
        assert response.num_clients == 0
        assert response.client_process_active
        assert response.producer_active
        assert response.clients_time_step_len_seconds == time_step_len_seconds
        assert response.producer_time_step_len_seconds == 30

    def test_startProducer(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startProducer grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        arrival_thread = mocker.MagicMock()
        arrival_thread.client_threads = ["client1", "client2"]
        arrival_thread.time_step_len_seconds = 30
        producer_thread = mocker.MagicMock()
        producer_thread.time_step_len_seconds = 30
        mocker.patch('time.sleep', return_value=None)
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_producer_thread',
                     return_value=producer_thread)
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_arrival_thread',
                     return_value=arrival_thread)
        mocker.patch('csle_collector.client_manager.threads.producer_thread.ProducerThread.__init__',
                     return_value=None)
        mocker.patch('csle_collector.client_manager.threads.producer_thread.ProducerThread.run',
                     return_value=True)
        mocker.patch('csle_collector.client_manager.threads.producer_thread.ProducerThread.start',
                     return_value=True)
        time_step_len_seconds = 30
        ip = "192.168.12.5"
        port = 3333
        response: ClientsDTO = csle_collector.client_manager.query_clients.start_producer(
            stub=grpc_stub, time_step_len_seconds=time_step_len_seconds, ip=ip, port=port)
        assert response is not None
        assert response.num_clients == len(arrival_thread.client_threads)
        assert response.client_process_active
        assert response.producer_active
        assert response.clients_time_step_len_seconds == 30
        assert response.producer_time_step_len_seconds == time_step_len_seconds

    def test_stopProducer(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopProducer grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        arrival_thread = mocker.MagicMock()
        arrival_thread.client_threads = ["client1", "client2"]
        arrival_thread.time_step_len_seconds = 30
        producer_thread = mocker.MagicMock()
        producer_thread.time_step_len_seconds = 30
        mocker.patch('time.sleep', return_value=None)
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_producer_thread',
                     return_value=producer_thread)
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_arrival_thread',
                     return_value=arrival_thread)
        response: ClientsDTO = csle_collector.client_manager.query_clients.stop_producer(stub=grpc_stub)
        assert response is not None
        assert response.num_clients == len(arrival_thread.client_threads)
        assert response.client_process_active
        assert not response.producer_active
        assert response.clients_time_step_len_seconds == arrival_thread.time_step_len_seconds
        assert response.producer_time_step_len_seconds == 0

        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_producer_thread',
                     return_value=None)
        mocker.patch('csle_collector.client_manager.client_manager.ClientManagerServicer.get_arrival_thread',
                     return_value=None)
        response = csle_collector.client_manager.query_clients.stop_clients(stub=grpc_stub)
        assert response is not None
        assert response.num_clients == 0
        assert not response.client_process_active
        assert not response.producer_active
        assert response.clients_time_step_len_seconds == 0.0
        assert response.producer_time_step_len_seconds == 0.0
